@@ -1,9 +1,10 @@
 package Net::DNS::Packet;
 
-use strict;
-use warnings;
-
-our $VERSION = (qw$Id: Packet.pm 2000 2025-01-02 16:34:13Z willem $)[2];
+#
+# $Id: Packet.pm 1381 2015-08-25 07:36:09Z willem $
+#
+use vars qw($VERSION);
+$VERSION = (qw$LastChangedRevision: 1381 $)[1];
 
 
 =head1 NAME
@@ -12,11 +13,11 @@ Net::DNS::Packet - DNS protocol packet
 
 =head1 SYNOPSIS
 
- use Net::DNS::Packet;
+    use Net::DNS::Packet;
 
- $query = Net::DNS::Packet->new( 'example.com', 'MX', 'IN' );
+    $query = new Net::DNS::Packet( 'example.com', 'MX', 'IN' );
 
- $reply = $resolver->send( $query );
+    $reply = $resolver->send( $query );
 
 
 =head1 DESCRIPTION
@@ -26,10 +27,10 @@ A Net::DNS::Packet object represents a DNS protocol packet.
 =cut
 
 
+use strict;
 use integer;
 use Carp;
 
-use Net::DNS::Parameters qw(:dsotype);
 use constant UDPSZ => 512;
 
 BEGIN {
@@ -43,10 +44,10 @@ BEGIN {
 
 =head2 new
 
- $packet = Net::DNS::Packet->new( 'example.com' );
- $packet = Net::DNS::Packet->new( 'example.com', 'MX', 'IN' );
+    $packet = new Net::DNS::Packet( 'example.com' );
+    $packet = new Net::DNS::Packet( 'example.com', 'MX', 'IN' );
 
- $packet = Net::DNS::Packet->new();
+    $packet = new Net::DNS::Packet();
 
 If passed a domain, type, and class, new() creates a Net::DNS::Packet
 object which is suitable for making a DNS query for the specified
@@ -58,8 +59,8 @@ If called with an empty argument list, new() creates an empty packet.
 =cut
 
 sub new {
-	my ( $class, @arg ) = @_;
-	return &decode if ref $arg[0];
+	return &decode if ref $_[1];
+	my $class = shift;
 
 	my $self = bless {
 		status	   => 0,
@@ -69,20 +70,21 @@ sub new {
 		additional => [],
 		}, $class;
 
-	$self->{question} = [Net::DNS::Question->new(@arg)] if scalar @arg;
+	$self->{question} = [Net::DNS::Question->new(@_)] if scalar @_;
 
 	return $self;
 }
 
 
-=head2 decode
+#=head2 decode
 
- $packet = Net::DNS::Packet->decode( \$data );
- $packet = Net::DNS::Packet->decode( \$data, 1 );	# debug
- $packet = Net::DNS::Packet->new( \$data ... );
+=pod
 
-A new packet object is created by decoding the DNS packet data
-contained in the scalar referenced by the first argument.
+    $packet = new Net::DNS::Packet( \$data );
+    $packet = new Net::DNS::Packet( \$data, 1 );	# debug
+
+If passed a reference to a scalar containing DNS packet data, a new
+packet object is created by decoding the data.
 The optional second boolean argument enables debugging output.
 
 Returns undef if unable to create a packet object.
@@ -91,7 +93,7 @@ Decoding errors, including data corruption and truncation, are
 collected in the $@ ($EVAL_ERROR) variable.
 
 
- ( $packet, $length ) = Net::DNS::Packet->decode( \$data );
+    ( $packet, $length ) = new Net::DNS::Packet( \$data );
 
 If called in array context, returns a packet object and the number
 of octets successfully decoded.
@@ -105,20 +107,19 @@ or corrupted during transmission.
 use constant HEADER_LENGTH => length pack 'n6', (0) x 6;
 
 sub decode {
-	my $class = shift;
+	my $class = shift;					# uncoverable pod
 	my $data  = shift;
 	my $debug = shift || 0;
 
 	my $offset = 0;
 	my $self;
 	eval {
-		local $SIG{__DIE__};
-		my $length = length $$data;
-		die 'corrupt wire-format data' if $length < HEADER_LENGTH;
+		die 'corrupt wire-format data' if length($$data) < HEADER_LENGTH;
 
 		# header section
 		my ( $id, $status, @count ) = unpack 'n6', $$data;
 		my ( $qd, $an, $ns, $ar ) = @count;
+		$offset = HEADER_LENGTH;
 
 		$self = bless {
 			id	   => $id,
@@ -128,60 +129,50 @@ sub decode {
 			answer	   => [],
 			authority  => [],
 			additional => [],
-			replysize  => $length
+			answersize => length $$data
 			}, $class;
 
 		# question/zone section
 		my $hash = {};
 		my $record;
-		$offset = HEADER_LENGTH;
 		while ( $qd-- ) {
-			( $record, $offset ) = Net::DNS::Question->decode( $data, $offset, $hash );
+			( $record, $offset ) = decode Net::DNS::Question( $data, $offset, $hash );
 			CORE::push( @{$self->{question}}, $record );
 		}
 
 		# RR sections
 		while ( $an-- ) {
-			( $record, $offset ) = Net::DNS::RR->decode( $data, $offset, $hash );
+			( $record, $offset ) = decode Net::DNS::RR( $data, $offset, $hash );
 			CORE::push( @{$self->{answer}}, $record );
 		}
 
 		while ( $ns-- ) {
-			( $record, $offset ) = Net::DNS::RR->decode( $data, $offset, $hash );
+			( $record, $offset ) = decode Net::DNS::RR( $data, $offset, $hash );
 			CORE::push( @{$self->{authority}}, $record );
 		}
 
 		while ( $ar-- ) {
-			( $record, $offset ) = Net::DNS::RR->decode( $data, $offset, $hash );
+			( $record, $offset ) = decode Net::DNS::RR( $data, $offset, $hash );
 			CORE::push( @{$self->{additional}}, $record );
 		}
 
-		return unless $offset == HEADER_LENGTH;
-		return unless $self->header->opcode eq 'DSO';
-
-		$self->{dso} = [];
-		my $limit = $length - 4;
-		while ( $offset < $limit ) {
-			my ( $t, $l, $v ) = unpack "\@$offset n2a*", $$data;
-			CORE::push( @{$self->{dso}}, [$t, substr( $v, 0, $l )] );
-			$offset += ( $l + 4 );
-		}
+		return $self;
 	};
 
 	if ($debug) {
 		local $@ = $@;
 		print $@ if $@;
-		eval { $self->print };
+		$self->print if $self;
 	}
 
 	return wantarray ? ( $self, $offset ) : $self;
 }
 
 
-=head2 encode
+=head2 data
 
- $data = $packet->encode;
- $data = $packet->encode( $size );
+    $data = $packet->data;
+    $data = $packet->data( $size );
 
 Returns the packet data in binary format, suitable for sending as a
 query or update request to a nameserver.
@@ -191,25 +182,26 @@ Truncation may be specified using a non-zero optional size argument.
 =cut
 
 sub data {
-	return &encode;						# uncoverable pod
+	&encode;
 }
 
 sub encode {
-	my ( $self, $size ) = @_;
+	my ( $self, $size ) = @_;				# uncoverable pod
 
 	my $edns = $self->edns;					# EDNS support
-	my @addl = grep { !$_->isa('Net::DNS::RR::OPT') } @{$self->{additional}};
-	$self->{additional} = [$edns, @addl] if $edns->_specified;
+	my @addl = grep !$_->isa('Net::DNS::RR::OPT'), @{$self->{additional}};
+	unshift( @addl, $edns ) if $edns->_specified;
+	$self->{additional} = \@addl;
 
 	return $self->truncate($size) if $size;
 
 	my @part = qw(question answer authority additional);
-	my @size = map { scalar @{$self->{$_}} } @part;
-	my $data = pack 'n6', $self->_quid, $self->{status}, @size;
+	my @size = map scalar( @{$self->{$_}} ), @part;
+	my $data = pack 'n6', $self->header->id, $self->{status}, @size;
 	$self->{count} = [];
 
 	my $hash = {};						# packet body
-	foreach my $component ( map { @{$self->{$_}} } @part ) {
+	foreach my $component ( map @{$self->{$_}}, @part ) {
 		$data .= $component->encode( length $data, $hash, $self );
 	}
 
@@ -219,7 +211,7 @@ sub encode {
 
 =head2 header
 
- $header = $packet->header;
+    $header = $packet->header;
 
 Constructor method which returns a Net::DNS::Header object which
 represents the header section of the packet.
@@ -228,14 +220,15 @@ represents the header section of the packet.
 
 sub header {
 	my $self = shift;
-	return bless \$self, q(Net::DNS::Header);
+	bless \$self, q(Net::DNS::Header);
 }
 
 
 =head2 edns
 
- $version = $packet->edns->version;
- $UDPsize = $packet->edns->size;
+    $edns    = $packet->edns;
+    $version = $edns->version;
+    $UDPsize = $edns->size;
 
 Auxiliary function which provides access to the EDNS protocol
 extension OPT RR.
@@ -245,15 +238,14 @@ extension OPT RR.
 sub edns {
 	my $self = shift;
 	my $link = \$self->{xedns};
-	($$link) = grep { $_->isa(qw(Net::DNS::RR::OPT)) } @{$self->{additional}} unless $$link;
-	$$link = Net::DNS::RR->new( type => 'OPT' ) unless $$link;
-	return $$link;
+	($$link) = grep $_->isa(qw(Net::DNS::RR::OPT)), @{$self->{additional}} unless $$link;
+	$$link = new Net::DNS::RR( type => 'OPT' ) unless $$link;
 }
 
 
 =head2 reply
 
- $reply = $query->reply( $UDPmax );
+    $reply = $query->reply( $UDPmax );
 
 Constructor method which returns a new reply packet.
 
@@ -264,11 +256,12 @@ response to an EDNS query.
 =cut
 
 sub reply {
-	my ( $query, @UDPmax ) = @_;
+	my $query  = shift;
+	my $UDPmax = shift;
 	my $qheadr = $query->header;
 	croak 'erroneous qr flag in query packet' if $qheadr->qr;
 
-	my $reply  = Net::DNS::Packet->new();
+	my $reply  = new Net::DNS::Packet();
 	my $header = $reply->header;
 	$header->qr(1);						# reply with same id, opcode and question
 	$header->id( $qheadr->id );
@@ -281,18 +274,14 @@ sub reply {
 	$header->rd( $qheadr->rd );				# copy these flags into reply
 	$header->cd( $qheadr->cd );
 
-	return $reply unless grep { $_->isa('Net::DNS::RR::OPT') } @{$query->{additional}};
-
-	my $edns = $reply->edns();
-	CORE::push( @{$reply->{additional}}, $edns );
-	$edns->udpsize(@UDPmax);
+	$reply->edns->size($UDPmax) if $query->edns->_specified;
 	return $reply;
 }
 
 
 =head2 question, zone
 
- @question = $packet->question;
+    @question = $packet->question;
 
 Returns a list of Net::DNS::Question objects representing the
 question section of the packet.
@@ -304,15 +293,14 @@ specifies the DNS zone to be updated.
 
 sub question {
 	my @qr = @{shift->{question}};
-	return @qr;
 }
 
-sub zone { return &question }
+sub zone {&question}
 
 
 =head2 answer, pre, prerequisite
 
- @answer = $packet->answer;
+    @answer = $packet->answer;
 
 Returns a list of Net::DNS::RR objects representing the answer
 section of the packet.
@@ -325,16 +313,15 @@ not preexist.
 
 sub answer {
 	my @rr = @{shift->{answer}};
-	return @rr;
 }
 
-sub pre		 { return &answer }
-sub prerequisite { return &answer }
+sub pre		 {&answer}
+sub prerequisite {&answer}
 
 
 =head2 authority, update
 
- @authority = $packet->authority;
+    @authority = $packet->authority;
 
 Returns a list of Net::DNS::RR objects representing the authority
 section of the packet.
@@ -346,15 +333,14 @@ specifies the RRs or RRsets to be added or deleted.
 
 sub authority {
 	my @rr = @{shift->{authority}};
-	return @rr;
 }
 
-sub update { return &authority }
+sub update {&authority}
 
 
 =head2 additional
 
- @additional = $packet->additional;
+    @additional = $packet->additional;
 
 Returns a list of Net::DNS::RR objects representing the additional
 section of the packet.
@@ -363,28 +349,24 @@ section of the packet.
 
 sub additional {
 	my @rr = @{shift->{additional}};
-	return @rr;
 }
 
 
 =head2 print
 
- $packet->print;
+    $packet->print;
 
-Prints the entire packet to the currently selected output filehandle
-using the master file format mandated by RFC1035.
+Prints the packet data on the standard output in an ASCII format
+similar to that used in DNS zone files.
 
 =cut
 
-sub print {
-	print &string;
-	return;
-}
+sub print { print &string; }
 
 
 =head2 string
 
- print $packet->string;
+    print $packet->string;
 
 Returns a string representation of the packet.
 
@@ -394,95 +376,81 @@ sub string {
 	my $self = shift;
 
 	my $header = $self->header;
-	my $opcode = $header->opcode;
-	my $packet = $header->qr ? 'Response' : 'Query';
-	my $server = $self->{replyfrom};
-	my $length = $self->{replysize};
-	my $origin = $server ? ";; $packet received from [$server] $length octets\n" : "";
-	my @record = ( "$origin;; HEADER SECTION", $header->string );
+	my $update = $header->opcode eq 'UPDATE';
 
-	if ( $opcode eq 'DSO' ) {
-		CORE::push( @record, ";; DSO SECTION" );
-		foreach ( @{$self->{dso}} ) {
-			my ( $t, $v ) = @$_;
-			CORE::push( @record, sprintf( ";;\t%s\t%s", dsotypebyval($t), unpack( 'H*', $v ) ) );
-		}
-		return join "\n", @record, "\n";
-	}
+	my $server = $self->{answerfrom};
+	my $length = $self->{answersize};
+	my $string = $server ? ";; Answer received from $server ($length bytes)\n" : "";
 
-	my $edns = $self->edns;
-	CORE::push( @record, $edns->string ) if $edns->_specified;
+	$string .= ";; HEADER SECTION\n" . $header->string;
 
-	my @section  = $opcode eq 'UPDATE' ? qw(ZONE PREREQUISITE UPDATE) : qw(QUESTION ANSWER AUTHORITY);
-	my @question = $self->question;
+	my $question = $update ? 'ZONE' : 'QUESTION';
+	my @question = map $_->string, $self->question;
 	my $qdcount  = scalar @question;
 	my $qds	     = $qdcount != 1 ? 's' : '';
-	CORE::push( @record, ";; $section[0] SECTION ($qdcount record$qds)", map { ';; ' . $_->string } @question );
+	$string .= join "\n;; ", "\n;; $question SECTION ($qdcount record$qds)", @question;
 
-	my @answer  = $self->answer;
+	my $answer = $update ? 'PREREQUISITE' : 'ANSWER';
+	my @answer  = map $_->string, $self->answer;
 	my $ancount = scalar @answer;
 	my $ans	    = $ancount != 1 ? 's' : '';
-	CORE::push( @record, "\n;; $section[1] SECTION ($ancount record$ans)", map { $_->string } @answer );
+	$string .= join "\n", "\n\n;; $answer SECTION ($ancount record$ans)", @answer;
 
-	my @authority = $self->authority;
+	my $authority = $update ? 'UPDATE' : 'AUTHORITY';
+	my @authority = map $_->string, $self->authority;
 	my $nscount   = scalar @authority;
 	my $nss	      = $nscount != 1 ? 's' : '';
-	CORE::push( @record, "\n;; $section[2] SECTION ($nscount record$nss)", map { $_->string } @authority );
+	$string .= join "\n", "\n\n;; $authority SECTION ($nscount record$nss)", @authority;
 
-	my @additional = $self->additional;
+	my @additional = map $_->string, $self->additional;
 	my $arcount    = scalar @additional;
 	my $ars	       = $arcount != 1 ? 's' : '';
-	my $EDNSmarker = join ' ', qq[;; {\t"EDNS-VERSION":], $edns->version, qq[}];
-	CORE::push( @record, "\n;; ADDITIONAL SECTION ($arcount record$ars)" );
-	CORE::push( @record, map { ( $_ eq $edns ) ? $EDNSmarker : $_->string } @additional );
+	$string .= join "\n", "\n\n;; ADDITIONAL SECTION ($arcount record$ars)", @additional;
 
-	return join "\n", @record, "\n";
+	return "$string\n\n";
 }
 
 
-=head2 from
+=head2 answerfrom
 
-    print "packet received from ", $packet->from, "\n";
+    print "packet received from ", $packet->answerfrom, "\n";
 
 Returns the IP address from which this packet was received.
-This method will return undef for user-created packets.
+User-created packets will return undef for this method.
 
 =cut
 
-sub from {
-	my ( $self, @argument ) = @_;
-	for (@argument) { $self->{replyfrom} = $_ }
-	return $self->{replyfrom};
+sub answerfrom {
+	my $self = shift;
+
+	$self->{answerfrom} = shift if scalar @_;
+	$self->{answerfrom};
 }
 
-sub answerfrom { return &from; }				# uncoverable pod
 
+=head2 answersize
 
-=head2 size
+    print "packet size: ", $packet->answersize, " bytes\n";
 
- print "packet size: ", $packet->size, " octets\n";
-
-Returns the size of the packet in octets as it was received from a
-nameserver.  This method will return undef for user-created packets
+Returns the size of the packet in bytes as it was received from a
+nameserver.  User-created packets will return undef for this method
 (use length($packet->data) instead).
 
 =cut
 
-sub size {
-	return shift->{replysize};
+sub answersize {
+	shift->{answersize};
 }
 
-sub answersize { return &size; }				# uncoverable pod
-
 
 =head2 push
 
- $ancount = $packet->push( prereq => $rr );
- $nscount = $packet->push( update => $rr );
- $arcount = $packet->push( additional => $rr );
+    $ancount = $packet->push( prereq => $rr );
+    $nscount = $packet->push( update => $rr );
+    $arcount = $packet->push( additional => $rr );
 
- $nscount = $packet->push( update => $rr1, $rr2, $rr3 );
- $nscount = $packet->push( update => @rr );
+    $nscount = $packet->push( update => $rr1, $rr2, $rr3 );
+    $nscount = $packet->push( update => @rr );
 
 Adds RRs to the specified section of the packet.
 
@@ -493,20 +461,20 @@ Section names may be abbreviated to the first three characters.
 =cut
 
 sub push {
-	my ( $self, $section, @rr ) = @_;
-	my $list = $self->_section($section);
-	return CORE::push( @$list, @rr );
+	my $self = shift;
+	my $list = $self->_section(shift);
+	CORE::push( @$list, grep ref($_), @_ );
 }
 
 
 =head2 unique_push
 
- $ancount = $packet->unique_push( prereq => $rr );
- $nscount = $packet->unique_push( update => $rr );
- $arcount = $packet->unique_push( additional => $rr );
+    $ancount = $packet->unique_push( prereq => $rr );
+    $nscount = $packet->unique_push( update => $rr );
+    $arcount = $packet->unique_push( additional => $rr );
 
- $nscount = $packet->unique_push( update => $rr1, $rr2, $rr3 );
- $nscount = $packet->unique_push( update => @rr );
+    $nscount = $packet->unique_push( update => $rr1, $rr2, $rr3 );
+    $nscount = $packet->unique_push( update => @rr );
 
 Adds RRs to the specified section of the packet provided that the
 RRs are not already present in the same section.
@@ -518,19 +486,21 @@ Section names may be abbreviated to the first three characters.
 =cut
 
 sub unique_push {
-	my ( $self, $section, @rr ) = @_;
-	my $list = $self->_section($section);
+	my $self = shift;
+	my $list = $self->_section(shift);
+	my @rr	 = grep ref($_), @_;
 
 	my %unique = map { ( bless( {%$_, ttl => 0}, ref $_ )->canonical => $_ ) } @rr, @$list;
-	return scalar( @$list = values %unique );
+
+	scalar( @$list = values %unique );
 }
 
 
 =head2 pop
 
- my $rr = $packet->pop( 'pre' );
- my $rr = $packet->pop( 'update' );
- my $rr = $packet->pop( 'additional' );
+    my $rr = $packet->pop( 'pre' );
+    my $rr = $packet->pop( 'update' );
+    my $rr = $packet->pop( 'additional' );
 
 Removes a single RR from the specified section of the packet.
 
@@ -539,7 +509,7 @@ Removes a single RR from the specified section of the packet.
 sub pop {
 	my $self = shift;
 	my $list = $self->_section(shift);
-	return CORE::pop(@$list);
+	CORE::pop(@$list);
 }
 
 
@@ -555,22 +525,22 @@ sub _section {				## returns array reference for section
 	my $self = shift;
 	my $name = shift;
 	my $list = $_section{unpack 'a3', $name} || $name;
-	return $self->{$list} ||= [];
+	$self->{$list} ||= [];
 }
 
 
 =head2 sign_tsig
 
- $query = Net::DNS::Packet->new( 'www.example.com', 'A' );
+    $query = Net::DNS::Packet->new( 'www.example.com', 'A' );
 
- $query->sign_tsig(
-		$keyfile,
+    $query->sign_tsig(
+		'Khmac-sha512.example.+165+01018.private',
 		fudge => 60
 		);
 
- $reply = $res->send( $query );
+    $reply = $res->send( $query );
 
- $reply->verify( $query ) || die $reply->verifyerr;
+    $reply->verify( $query ) || die $reply->verifyerr;
 
 Attaches a TSIG resource record object, which will be used to sign
 the packet (see RFC 2845).
@@ -584,7 +554,7 @@ must uniquely identify the key shared between the parties, and the
 algorithm name must identify the signing function to be used with the
 specified key.
 
- $tsig = Net::DNS::RR->new(
+    $tsig = Net::DNS::RR->new(
 		name		=> 'tsig.example',
 		type		=> 'TSIG',
 		algorithm	=> 'custom-algorithm',
@@ -595,22 +565,28 @@ specified key.
 					}
 		);
 
- $query->sign_tsig( $tsig );
+    $query->sign_tsig( $tsig );
+
+
+The historical simplified syntax is still available, but additional
+options can not be specified.
+
+    $packet->sign_tsig( $key_name, $key );
 
 
 The response to an inbound request is signed by presenting the request
 in place of the key parameter.
 
- $response = $request->reply;
- $response->sign_tsig( $request, @options );
+    $response = $request->reply;
+    $response->sign_tsig( $request, @options );
 
 
 Multi-packet transactions are signed by chaining the sign_tsig()
 calls together as follows:
 
- $opaque = $packet1->sign_tsig( 'Kexample.+165+13281.private' );
- $opaque = $packet2->sign_tsig( $opaque );
- $opaque = $packet3->sign_tsig( $opaque );
+    $opaque  =	$packet1->sign_tsig( 'Kexample.+165+13281.private' );
+    $opaque  =	$packet2->sign_tsig( $opaque );
+		$packet3->sign_tsig( $opaque );
 
 The opaque intermediate object references returned during multi-packet
 signing are not intended to be accessed by the end-user application.
@@ -622,28 +598,30 @@ does not support the suppressed signature scheme described in RFC2845.
 =cut
 
 sub sign_tsig {
-	my ( $self, @argument ) = @_;
-	return eval {
-		local $SIG{__DIE__};
+	my $self = shift;
+
+	eval {
 		require Net::DNS::RR::TSIG;
-		my $tsig = Net::DNS::RR::TSIG->create(@argument);
+		my $tsig = Net::DNS::RR::TSIG->create(@_);
 		$self->push( 'additional' => $tsig );
 		return $tsig;
-	} || return croak "$@\nTSIG: unable to sign packet";
+	} || do {
+		croak "$@\nTSIG: unable to sign packet";
+	};
 }
 
 
 =head2 verify and verifyerr
 
- $packet->verify()	|| die $packet->verifyerr;
- $reply->verify($query)	|| die $reply->verifyerr;
+    $packet->verify()		|| die $packet->verifyerr;
+    $reply->verify( $query )	|| die $reply->verifyerr;
 
 Verify TSIG signature of packet or reply to the corresponding query.
 
 
- $opaque  = $packet1->verify( $query ) || die $packet1->verifyerr;
- $opaque  = $packet2->verify( $opaque );
- $verifed = $packet3->verify( $opaque ) || die $packet3->verifyerr;
+    $opaque  =	$packet1->verify( $query ) || die $packet1->verifyerr;
+    $opaque  =	$packet2->verify( $opaque );
+    $verifed =	$packet3->verify( $opaque ) || die $packet3->verifyerr;
 
 The opaque intermediate object references returned during multi-packet
 verify() will be undefined (Boolean false) if verification fails.
@@ -654,13 +632,16 @@ on the final packet in the absence of more specific information.
 =cut
 
 sub verify {
-	my ( $self, @argument ) = @_;
+	my $self = shift;
+
 	my $sig = $self->sigrr;
-	return $sig ? $sig->verify( $self, @argument ) : shift @argument;
+	return $sig ? $sig->verify( $self, @_ ) : undef;
 }
 
 sub verifyerr {
-	my $sig = shift->sigrr;
+	my $self = shift;
+
+	my $sig = $self->sigrr;
 	return $sig ? $sig->vrfyerrstr : 'not signed';
 }
 
@@ -672,17 +653,17 @@ The requisite cryptographic components are not integrated into
 Net::DNS but reside in the Net::DNS::SEC distribution available
 from CPAN.
 
- $update = Net::DNS::Update->new('example.com');
- $update->push( update => rr_add('foo.example.com A 10.1.2.3'));
- $update->sign_sig0('Kexample.com+003+25317.private');
+    $update = new Net::DNS::Update('example.com');
+    $update->push( update => rr_add('foo.example.com A 10.1.2.3'));
+    $update->sign_sig0('Kexample.com+003+25317.private');
 
 Execution will be terminated if Net::DNS::SEC is not available.
 
 
 =head2 verify SIG0
 
- $packet->verify( $keyrr )		|| die $packet->verifyerr;
- $packet->verify( [$keyrr, ...] )	|| die $packet->verifyerr;
+    $packet->verify( $keyrr )		|| die $packet->verifyerr;
+    $packet->verify( [$keyrr, ...] )	|| die $packet->verifyerr;
 
 Verify SIG0 packet signature against one or more specified KEY RRs.
 
@@ -692,27 +673,28 @@ sub sign_sig0 {
 	my $self = shift;
 	my $karg = shift;
 
-	return eval {
-		local $SIG{__DIE__};
-
+	eval {
 		my $sig0;
+		require Net::DNS::RR::SIG;
+
 		if ( ref($karg) eq 'Net::DNS::RR::SIG' ) {
 			$sig0 = $karg;
 
 		} else {
-			require Net::DNS::RR::SIG;
 			$sig0 = Net::DNS::RR::SIG->create( '', $karg );
 		}
 
 		$self->push( 'additional' => $sig0 );
 		return $sig0;
-	} || return croak "$@\nSIG0: unable to sign packet";
+	} || do {
+		croak "$@\nSIG0: unable to sign packet";
+	};
 }
 
 
 =head2 sigrr
 
- $sigrr = $packet->sigrr() || die 'unsigned packet';
+    $sigrr = $packet->sigrr() || die 'unsigned packet';
 
 The sigrr method returns the signature RR from a signed packet
 or undefined if the signature is absent.
@@ -723,11 +705,10 @@ sub sigrr {
 	my $self = shift;
 
 	my ($sig) = reverse $self->additional;
-	return unless $sig;
-	for ( $sig->type ) {
-		return $sig if /TSIG|SIG/;
-	}
-	return;
+	return undef unless $sig;
+	return $sig if $sig->type eq 'TSIG';
+	return $sig if $sig->type eq 'SIG';
+	return undef;
 }
 
 
@@ -739,7 +720,7 @@ The truncate method takes a maximum length as argument and then tries
 to truncate the packet and set the TC bit according to the rules of
 RFC2181 Section 9.
 
-The smallest length limit that is honoured is 512 octets.
+The minimum maximum length that is honoured is 512 octets.
 
 =cut
 
@@ -772,12 +753,13 @@ sub truncate {
 	$size = UDPSZ unless $size > UDPSZ;
 	$size -= $sigrr->_size if $sigrr;
 
-	my $data = pack 'x' x HEADER_LENGTH;			# header placeholder
+	my $data = pack 'x' x 12;				# header placeholder
+	my $hdsz = length $data;
 	$self->{count} = [];
 
 	my $tc;
 	my $hash = {};
-	foreach my $section ( map { $self->{$_} } qw(question answer authority) ) {
+	foreach my $section ( map $self->{$_}, qw(question answer authority) ) {
 		my @list;
 		foreach my $item (@$section) {
 			my $component = $item->encode( length $data, $hash );
@@ -791,18 +773,18 @@ sub truncate {
 	}
 	$self->header->tc(1) if $tc;				# only set if truncated here
 
+	my @list;
+	my @rrset;
 	my %rrset;
-	my @order;
-	foreach my $item ( grep { ref($_) ne ref($sigrr) } $self->additional ) {
+	foreach my $item ( grep ref($_) ne ref($sigrr), @{$self->{additional}} ) {
 		my $name  = $item->{owner}->canonical;
 		my $class = $item->{class} || 0;
 		my $key	  = pack 'nna*', $class, $item->{type}, $name;
-		CORE::push @order, $key unless $rrset{$key};
+		CORE::push @rrset, $key unless $rrset{$key};
 		CORE::push @{$rrset{$key}}, $item;
 	}
 
-	my @list;
-	foreach my $key (@order) {
+	foreach my $key (@rrset) {
 		my $component = '';
 		my @item      = @{$rrset{$key}};
 		foreach my $item (@item) {
@@ -817,38 +799,21 @@ sub truncate {
 		$data .= $sigrr->encode( length $data, $hash, $self );
 		CORE::push @list, $sigrr;
 	}
-	$self->{'additional'} = \@list;
+	$self->{additional} = \@list;
 
 	my @part = qw(question answer authority additional);
-	my @size = map { scalar @{$self->{$_}} } @part;
-	return pack 'n6 a*', $self->_quid, $self->{status}, @size, substr( $data, HEADER_LENGTH );
+	my @size = map scalar( @{$self->{$_}} ), @part;
+	pack 'n6 a*', $self->header->id, $self->{status}, @size, substr( $data, $hdsz );
 }
 
 
 ########################################
 
 sub dump {				## print internal data structure
-	my @data = @_;						# uncoverable pod
-	require Data::Dumper;
+	require Data::Dumper;					# uncoverable pod
 	local $Data::Dumper::Maxdepth = $Data::Dumper::Maxdepth || 3;
 	local $Data::Dumper::Sortkeys = $Data::Dumper::Sortkeys || 1;
-	local $Data::Dumper::Useqq    = $Data::Dumper::Useqq	|| 1;
-	print Data::Dumper::Dumper(@data);
-	return;
-}
-
-
-my ( $cache1, $cache2, $limit );
-
-sub _quid {				## generate (short-term) unique query ID
-	my $self = shift;
-	my $id	 = $self->{id};
-	$cache1->{$id}++ if $id;				# cache non-zero ID
-	return $id	 if defined $id;
-	( $cache2, $cache1, $limit ) = ( $cache1, {0 => 1}, 50 ) unless $limit--;
-	$id = int rand(0xffff);					# two layer ID cache
-	$id = int rand(0xffff) while $cache1->{$id}++ + exists( $cache2->{$id} );
-	return $self->{id} = $id;
+	print Data::Dumper::Dumper(@_);
 }
 
 
@@ -864,7 +829,7 @@ Portions Copyright (c)2002-2004 Chris Reinhardt.
 
 Portions Copyright (c)2002-2009 Olaf Kolkman
 
-Portions Copyright (c)2007-2019 Dick Franks
+Portions Copyright (c)2007-2015 Dick Franks
 
 All rights reserved.
 
@@ -873,7 +838,7 @@ All rights reserved.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the original copyright notices appear in all copies and that both
+that the above copyright notice appear in all copies and that both that
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
@@ -890,12 +855,9 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl> L<Net::DNS> L<Net::DNS::Header>
-L<Net::DNS::Question> L<Net::DNS::RR> L<Net::DNS::RR::TSIG>
-L<RFC1035(4.1)|https://iana.org/go/rfc1035#section-4.1>
-L<RFC2136|https://iana.org/go/rfc2136#section-2>
-L<RFC8490|https://iana.org/go/rfc8490>
-L<RFC8945|https://iana.org/go/rfc8945>
+L<perl>, L<Net::DNS>, L<Net::DNS::Update>, L<Net::DNS::Header>,
+L<Net::DNS::Question>, L<Net::DNS::RR>, L<Net::DNS::RR::TSIG>,
+RFC1035 Section 4.1, RFC2136 Section 2, RFC2845
 
 =cut
 
