@@ -1,13 +1,13 @@
 package Net::DNS::RR::A;
 
 #
-# $Id: A.pm 1528 2017-01-18 21:44:58Z willem $
+# $Id: A.pm 1388 2015-09-07 12:28:16Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1528 $)[1];
+use vars qw($VERSION);
+$VERSION = (qw$LastChangedRevision: 1388 $)[1];
 
 
 use strict;
-use warnings;
 use base qw(Net::DNS::RR);
 
 =head1 NAME
