@@ -1,10 +1,10 @@
 package Net::DNS::RR::AFSDB;
 
 #
-# $Id: AFSDB.pm 1362 2015-06-23 08:47:14Z willem $
+# $Id: AFSDB.pm 1390 2015-09-11 11:42:11Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1362 $)[1];
+$VERSION = (qw$LastChangedRevision: 1390 $)[1];
 
 
 use strict;
@@ -22,7 +22,7 @@ use integer;
 use Net::DNS::DomainName;
 
 
-sub decode_rdata {			## decode rdata from wire-format octet string
+sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 	my ( $data, $offset, @opaque ) = @_;
 
@@ -31,25 +31,24 @@ sub decode_rdata {			## decode rdata from wire-format octet string
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 	my ( $offset, @opaque ) = @_;
 
-	return '' unless $self->{hostname};
-	my $rdata = pack 'n', $self->subtype;
-	$rdata .= $self->{hostname}->encode( $offset + length($rdata), @opaque );
+	my $hostname = $self->{hostname} || return '';
+	pack 'n a*', $self->subtype, $hostname->encode( $offset + 2, @opaque );
 }
 
 
-sub format_rdata {			## format rdata portion of RR string.
+sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless $self->{hostname};
-	join ' ', $self->subtype, $self->{hostname}->string;
+	my $hostname = $self->{hostname} || return '';
+	join ' ', $self->subtype, $hostname->string;
 }
 
 
-sub parse_rdata {			## populate RR from rdata in argument list
+sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
 	$self->subtype(shift);
