@@ -1,10 +1,10 @@
 package Net::DNS::RR::CAA;
 
 #
-# $Id: CAA.pm 1362 2015-06-23 08:47:14Z willem $
+# $Id: CAA.pm 1390 2015-09-11 11:42:11Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1362 $)[1];
+$VERSION = (qw$LastChangedRevision: 1390 $)[1];
 
 
 use strict;
@@ -22,7 +22,7 @@ use integer;
 use Net::DNS::Text;
 
 
-sub decode_rdata {			## decode rdata from wire-format octet string
+sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 	my ( $data, $offset ) = @_;
 
@@ -33,7 +33,7 @@ sub decode_rdata {			## decode rdata from wire-format octet string
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
 	my $tag = $self->{tag} || return '';
@@ -41,15 +41,15 @@ sub encode_rdata {			## encode rdata as wire-format octet string
 }
 
 
-sub format_rdata {			## format rdata portion of RR string.
+sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
 	my $tag = $self->{tag} || return '';
-	my @rdata = $self->flags, $tag->string, $self->{value}->string;
+	my @rdata = ( $self->flags, $tag->string, $self->{value}->string );
 }
 
 
-sub parse_rdata {			## populate RR from rdata in argument list
+sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
 	$self->flags(shift);
@@ -58,7 +58,7 @@ sub parse_rdata {			## populate RR from rdata in argument list
 }
 
 
-sub defaults() {			## specify RR attribute default values
+sub _defaults {				## specify RR attribute default values
 	my $self = shift;
 
 	$self->flags(0);
