@@ -1,10 +1,10 @@
 package Net::DNS::RR::CERT;
 
 #
-# $Id: CERT.pm 1362 2015-06-23 08:47:14Z willem $
+# $Id: CERT.pm 1390 2015-09-11 11:42:11Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1362 $)[1];
+$VERSION = (qw$LastChangedRevision: 1390 $)[1];
 
 
 use strict;
@@ -67,30 +67,29 @@ my %certtype = (
 
 	my $map = sub {
 		my $arg = shift;
-		unless ( $algbyval{$arg} ) {
-			$arg =~ s/[^A-Za-z0-9]//g;		# synthetic key
-			return uc $arg;
-		}
-		my @map = ( $arg, "$arg" => $arg );		# also accept number
+		return $arg if $arg =~ /^\d/;
+		$arg =~ s/[^A-Za-z0-9]//g;			# strip non-alphanumerics
+		uc($arg);
 	};
 
-	my %algbyname = map &$map($_), @algbyname;
+	my @pairedval = sort ( 1 .. 254, 1 .. 254 );		# also accept number
+	my %algbyname = map &$map($_), @algbyname, @pairedval;
 
-	sub algbyname {
+	sub _algbyname {
 		my $name = shift;
 		my $key	 = uc $name;				# synthetic key
 		$key =~ s/[^A-Z0-9]//g;				# strip non-alphanumerics
 		$algbyname{$key} || croak "unknown algorithm $name";
 	}
 
-	sub algbyval {
+	sub _algbyval {
 		my $value = shift;
 		$algbyval{$value} || return $value;
 	}
 }
 
 
-sub decode_rdata {			## decode rdata from wire-format octet string
+sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 	my ( $data, $offset ) = @_;
 
@@ -99,23 +98,24 @@ sub decode_rdata {			## decode rdata from wire-format octet string
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	my $certbin = $self->certbin || return '';
-	pack "n2 C a*", $self->certtype, $self->keytag, $self->algorithm, $certbin;
+	return '' unless defined $self->{certbin};
+	pack "n2 C a*", $self->certtype, $self->keytag, $self->algorithm, $self->{certbin};
 }
 
 
-sub format_rdata {			## format rdata portion of RR string.
+sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	my @base64 = split /\s+/, encode_base64( $self->certbin || return '' );
-	my @rdata = $self->certtype, $self->keytag, $self->algorithm, @base64;
+	return '' unless defined $self->{certbin};
+	my @base64 = split /\s+/, encode_base64( $self->{certbin} );
+	my @rdata = ( $self->certtype, $self->keytag, $self->algorithm, @base64 );
 }
 
 
-sub parse_rdata {			## populate RR from rdata in argument list
+sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
 	$self->certtype(shift);
@@ -151,8 +151,8 @@ sub algorithm {
 	my ( $self, $arg ) = @_;
 
 	return $self->{algorithm} unless defined $arg;
-	return algbyval( $self->{algorithm} ) if $arg =~ /MNEMONIC/i;
-	return $self->{algorithm} = $arg ? algbyname($arg) : 0;
+	return _algbyval( $self->{algorithm} ) if $arg =~ /MNEMONIC/i;
+	return $self->{algorithm} = $arg ? _algbyname($arg) : 0;
 }
 
 
@@ -175,8 +175,9 @@ sub cert {
 }
 
 
-sub format { &certtype; }		## historical
-sub tag	   { &keytag; }			## historical
+sub format { &certtype; }					# uncoverable pod
+
+sub tag { &keytag; }						# uncoverable pod
 
 
 1;
