@@ -1,10 +1,10 @@
 package Net::DNS::RR::CNAME;
 
 #
-# $Id: CNAME.pm 1348 2015-05-27 09:55:13Z willem $
+# $Id: CNAME.pm 1390 2015-09-11 11:42:11Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1348 $)[1];
+$VERSION = (qw$LastChangedRevision: 1390 $)[1];
 
 
 use strict;
@@ -22,30 +22,30 @@ use integer;
 use Net::DNS::DomainName;
 
 
-sub decode_rdata {			## decode rdata from wire-format octet string
+sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 
 	$self->{cname} = decode Net::DNS::DomainName1035(@_);
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	return '' unless $self->{cname};
-	$self->{cname}->encode(@_);
+	my $cname = $self->{cname} || return '';
+	$cname->encode(@_);
 }
 
 
-sub format_rdata {			## format rdata portion of RR string.
+sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless $self->{cname};
-	$self->{cname}->string;
+	my $cname = $self->{cname} || return '';
+	$cname->string;
 }
 
 
-sub parse_rdata {			## populate RR from rdata in argument list
+sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
 	$self->cname(shift);
