@@ -1,10 +1,10 @@
 package Net::DNS::RR::DNAME;
 
 #
-# $Id: DNAME.pm 1347 2015-05-19 10:24:36Z willem $
+# $Id: DNAME.pm 1390 2015-09-11 11:42:11Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1347 $)[1];
+$VERSION = (qw$LastChangedRevision: 1390 $)[1];
 
 
 use strict;
@@ -22,30 +22,30 @@ use integer;
 use Net::DNS::DomainName;
 
 
-sub decode_rdata {			## decode rdata from wire-format octet string
+sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 
 	$self->{target} = decode Net::DNS::DomainName2535(@_);
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	return '' unless $self->{target};
-	$self->{target}->encode(@_);
+	my $target = $self->{target} || return '';
+	$target->encode(@_);
 }
 
 
-sub format_rdata {			## format rdata portion of RR string.
+sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless $self->{target};
-	$self->{target}->string;
+	my $target = $self->{target} || return '';
+	$target->string;
 }
 
 
-sub parse_rdata {			## populate RR from rdata in argument list
+sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
 	$self->target(shift);
@@ -60,7 +60,7 @@ sub target {
 }
 
 
-sub dname { &target; }			## historical
+sub dname { &target; }						# uncoverable pod
 
 
 1;
