@@ -1,10 +1,10 @@
 package Net::DNS::RR::DNSKEY;
 
 #
-# $Id: DNSKEY.pm 1367 2015-06-29 08:53:56Z willem $
+# $Id: DNSKEY.pm 1390 2015-09-11 11:42:11Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1367 $)[1];
+$VERSION = (qw$LastChangedRevision: 1390 $)[1];
 
 
 use strict;
@@ -52,30 +52,29 @@ use MIME::Base64;
 
 	my $map = sub {
 		my $arg = shift;
-		unless ( $algbyval{$arg} ) {
-			$arg =~ s/[^A-Za-z0-9]//g;		# synthetic key
-			return uc $arg;
-		}
-		my @map = ( $arg, "$arg" => $arg );		# also accept number
+		return $arg if $arg =~ /^\d/;
+		$arg =~ s/[^A-Za-z0-9]//g;			# strip non-alphanumerics
+		uc($arg);
 	};
 
-	my %algbyname = map &$map($_), @algbyname;
+	my @pairedval = sort ( 1 .. 254, 1 .. 254 );		# also accept number
+	my %algbyname = map &$map($_), @algbyname, @pairedval;
 
-	sub algbyname {
+	sub _algbyname {
 		my $name = shift;
 		my $key	 = uc $name;				# synthetic key
 		$key =~ s/[^A-Z0-9]//g;				# strip non-alphanumerics
 		$algbyname{$key} || croak "unknown algorithm $name";
 	}
 
-	sub algbyval {
+	sub _algbyval {
 		my $value = shift;
 		$algbyval{$value} || return $value;
 	}
 }
 
 
-sub decode_rdata {			## decode rdata from wire-format octet string
+sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 	my ( $data, $offset ) = @_;
 
@@ -84,33 +83,34 @@ sub decode_rdata {			## decode rdata from wire-format octet string
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	my $keybin = $self->keybin || return '';
-	pack 'n C2 a*', $self->flags, $self->protocol, $self->algorithm, $keybin;
+	return '' unless defined $self->{keybin};
+	pack 'n C2 a*', $self->flags, $self->protocol, $self->algorithm, $self->{keybin};
 }
 
 
-sub format_rdata {			## format rdata portion of RR string.
+sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	my @base64 = split /\s+/, encode_base64( $self->keybin || return '' );
+	return '' unless defined $self->{keybin};
+	my @base64 = split /\s+/, encode_base64( $self->{keybin} );
 	my $keytag = $self->keytag;
 	my @params = map $self->$_, qw(flags protocol algorithm);
-	my @rdata  = @params, @base64, "; Key ID = $keytag\n";
+	my @rdata  = ( @params, @base64, "; Key ID = $keytag\n" );
 }
 
 
-sub parse_rdata {			## populate RR from rdata in argument list
+sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
-	$self->$_(shift) for qw(flags protocol algorithm);
+	foreach (qw(flags protocol algorithm)) { $self->$_(shift) }
 	$self->key(@_);
 }
 
 
-sub defaults() {			## specify RR attribute default values
+sub _defaults {				## specify RR attribute default values
 	my $self = shift;
 
 	$self->algorithm(1);
@@ -173,12 +173,12 @@ sub algorithm {
 
 	unless ( ref($self) ) {		## class method or simple function
 		my $argn = pop;
-		return $argn =~ /[^0-9]/ ? algbyname($argn) : algbyval($argn);
+		return $argn =~ /[^0-9]/ ? _algbyname($argn) : _algbyval($argn);
 	}
 
 	return $self->{algorithm} unless defined $arg;
-	return algbyval( $self->{algorithm} ) if $arg =~ /MNEMONIC/i;
-	return $self->{algorithm} = algbyname($arg);
+	return _algbyval( $self->{algorithm} ) if $arg =~ /MNEMONIC/i;
+	return $self->{algorithm} = _algbyname($arg);
 }
 
 
@@ -207,6 +207,7 @@ sub privatekeyname {
 	sprintf 'K%s+%03d+%05d.private', $name, $self->algorithm, $self->keytag;
 }
 
+
 sub signame {
 	my $self = shift;
 	my $name = lc $self->{owner}->fqdn;
@@ -218,7 +219,7 @@ sub keylength {
 
 	my $keybin = $self->keybin || return undef;
 
-	local $_ = algbyval( $self->{algorithm} );
+	local $_ = _algbyval( $self->{algorithm} );
 
 	if (/^RSA/) {
 
@@ -350,6 +351,8 @@ The 8-bit algorithm number describes the public key algorithm.
 algorithm() may also be invoked as a class method or simple function
 to perform mnemonic and numeric code translation.
 
+=head2 publickey
+
 =head2 key
 
     $key = $rr->key;
@@ -366,13 +369,17 @@ Opaque octet string representing the public key material.
 
 =head2 privatekeyname
 
-    $privatekeyname = $rr->privatekeyname
+    $privatekeyname = $rr->privatekeyname;
 
 Returns the name of the privatekey as it would be generated by
 the BIND dnssec-keygen program. The format of that name being:
 
 	K<fqdn>+<algorithm>+<keyid>.private
 
+=head2 signame
+
+Returns the canonical signer name of the privatekey.
+
 =head2 keylength
 
 Returns the length (in bits) of the modulus calculated from the key text.
