@@ -1,10 +1,10 @@
 package Net::DNS::RR::ISDN;
 
 #
-# $Id: ISDN.pm 1382 2015-08-26 20:39:33Z willem $
+# $Id: ISDN.pm 1390 2015-09-11 11:42:11Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1382 $)[1];
+$VERSION = (qw$LastChangedRevision: 1390 $)[1];
 
 
 use strict;
@@ -34,8 +34,8 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	return '' unless defined $self->{address};
-	join '', $self->{address}->encode, $self->{sa}->encode;
+	my $address = $self->{address} || return '';
+	join '', $address->encode, $self->{sa}->encode;
 }
 
 
@@ -106,7 +106,6 @@ other unpredictable behaviour.
 
 =head2 ISDNaddress
 
-
 =head2 address
 
     $address = $rr->address;
