@@ -1,10 +1,10 @@
 package Net::DNS::RR::KX;
 
 #
-# $Id: KX.pm 1362 2015-06-23 08:47:14Z willem $
+# $Id: KX.pm 1390 2015-09-11 11:42:11Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1362 $)[1];
+$VERSION = (qw$LastChangedRevision: 1390 $)[1];
 
 
 use strict;
@@ -22,7 +22,7 @@ use integer;
 use Net::DNS::DomainName;
 
 
-sub decode_rdata {			## decode rdata from wire-format octet string
+sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 	my ( $data, $offset, @opaque ) = @_;
 
@@ -31,25 +31,24 @@ sub decode_rdata {			## decode rdata from wire-format octet string
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 	my ( $offset, @opaque ) = @_;
 
-	return '' unless $self->{exchange};
-	my $rdata = pack 'n', $self->preference;
-	$rdata .= $self->{exchange}->encode( $offset + length($rdata), @opaque );
+	my $exchange = $self->{exchange} || return '';
+	pack 'n a*', $self->preference, $exchange->encode( $offset + 2, @opaque );
 }
 
 
-sub format_rdata {			## format rdata portion of RR string.
+sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless $self->{exchange};
-	join ' ', $self->preference, $self->{exchange}->string;
+	my $exchange = $self->{exchange} || return '';
+	join ' ', $self->preference, $exchange->string;
 }
 
 
-sub parse_rdata {			## populate RR from rdata in argument list
+sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
 	$self->preference(shift);
@@ -73,13 +72,13 @@ sub exchange {
 }
 
 
-__PACKAGE__->set_rrsort_func(		## sort RRs in numerically ascending order.
-	'preference',
-	sub { $Net::DNS::a->{'preference'} <=> $Net::DNS::b->{'preference'} }
+my $function = sub {			## sort RRs in numerically ascending order.
+	$Net::DNS::a->{'preference'} <=> $Net::DNS::b->{'preference'};
+};
 
-	);
+__PACKAGE__->set_rrsort_func( 'preference', $function );
 
-__PACKAGE__->set_rrsort_func( 'default_sort', __PACKAGE__->get_rrsort_func('preference') );
+__PACKAGE__->set_rrsort_func( 'default_sort', $function );
 
 
 1;
