@@ -1,10 +1,10 @@
 package Net::DNS::RR::MINFO;
 
 #
-# $Id: MINFO.pm 1348 2015-05-27 09:55:13Z willem $
+# $Id: MINFO.pm 1390 2015-09-11 11:42:11Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1348 $)[1];
+$VERSION = (qw$LastChangedRevision: 1390 $)[1];
 
 
 use strict;
@@ -22,7 +22,7 @@ use integer;
 use Net::DNS::Mailbox;
 
 
-sub decode_rdata {			## decode rdata from wire-format octet string
+sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 	my ( $data, $offset, @opaque ) = @_;
 
@@ -31,25 +31,25 @@ sub decode_rdata {			## decode rdata from wire-format octet string
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 	my ( $offset, @opaque ) = @_;
 
-	return '' unless $self->{emailbx};
+	my $emailbx = $self->{emailbx} || return '';
 	my $rdata = $self->{rmailbx}->encode(@_);
-	$rdata .= $self->{emailbx}->encode( $offset + length $rdata, @opaque );
+	$rdata .= $emailbx->encode( $offset + length $rdata, @opaque );
 }
 
 
-sub format_rdata {			## format rdata portion of RR string.
+sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless $self->{emailbx};
-	join ' ', map $self->{$_}->string, qw(rmailbx emailbx);
+	my $emailbx = $self->{emailbx} || return '';
+	my @rdata = ( $self->{rmailbx}->string, $emailbx->string );
 }
 
 
-sub parse_rdata {			## populate RR from rdata in argument list
+sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
 	$self->rmailbx(shift);
