@@ -1,10 +1,10 @@
 package Net::DNS::RR::MX;
 
 #
-# $Id: MX.pm 1362 2015-06-23 08:47:14Z willem $
+# $Id: MX.pm 1390 2015-09-11 11:42:11Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1362 $)[1];
+$VERSION = (qw$LastChangedRevision: 1390 $)[1];
 
 
 use strict;
@@ -22,7 +22,7 @@ use integer;
 use Net::DNS::DomainName;
 
 
-sub decode_rdata {			## decode rdata from wire-format octet string
+sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 	my ( $data, $offset, @opaque ) = @_;
 
@@ -31,17 +31,16 @@ sub decode_rdata {			## decode rdata from wire-format octet string
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 	my ( $offset, @opaque ) = @_;
 
 	my $exchange = $self->{exchange} || return '';
-	my $rdata = pack 'n', $self->preference;
-	$rdata .= $exchange->encode( $offset + length($rdata), @opaque );
+	pack 'n a*', $self->preference, $exchange->encode( $offset + 2, @opaque );
 }
 
 
-sub format_rdata {			## format rdata portion of RR string.
+sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
 	my $exchange = $self->{exchange} || return '';
@@ -49,7 +48,7 @@ sub format_rdata {			## format rdata portion of RR string.
 }
 
 
-sub parse_rdata {			## populate RR from rdata in argument list
+sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
 	$self->preference(shift);
@@ -57,7 +56,7 @@ sub parse_rdata {			## populate RR from rdata in argument list
 }
 
 
-sub defaults() {			## specify RR attribute default values
+sub _defaults {				## specify RR attribute default values
 	my $self = shift;
 
 	$self->preference(10);
@@ -80,14 +79,14 @@ sub exchange {
 }
 
 
-eval {					## avoid compilation failure using ancient perl
-
-	# sort RRs in numerically ascending order.
-	my $function = __PACKAGE__->get_rrsort_func('preference');
-
-	__PACKAGE__->set_rrsort_func( 'default_sort', $function );
+my $function = sub {			## sort RRs in numerically ascending order.
+	$Net::DNS::a->{'preference'} <=> $Net::DNS::b->{'preference'};
 };
 
+__PACKAGE__->set_rrsort_func( 'preference', $function );
+
+__PACKAGE__->set_rrsort_func( 'default_sort', $function );
+
 
 1;
 __END__
