@@ -1,10 +1,10 @@
 package Net::DNS::RR::NAPTR;
 
 #
-# $Id: NAPTR.pm 1362 2015-06-23 08:47:14Z willem $
+# $Id: NAPTR.pm 1390 2015-09-11 11:42:11Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1362 $)[1];
+$VERSION = (qw$LastChangedRevision: 1390 $)[1];
 
 
 use strict;
@@ -23,7 +23,7 @@ use Net::DNS::DomainName;
 use Net::DNS::Text;
 
 
-sub decode_rdata {			## decode rdata from wire-format octet string
+sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 	my ( $data, $offset, @opaque ) = @_;
 
@@ -35,11 +35,11 @@ sub decode_rdata {			## decode rdata from wire-format octet string
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 	my ( $offset, @opaque ) = @_;
 
-	return '' unless $self->{replacement};
+	return '' unless defined $self->{replacement};
 	my $rdata = pack 'n2', @{$self}{qw(order preference)};
 	$rdata .= $self->{flags}->encode;
 	$rdata .= $self->{service}->encode;
@@ -48,20 +48,19 @@ sub encode_rdata {			## encode rdata as wire-format octet string
 }
 
 
-sub format_rdata {			## format rdata portion of RR string.
+sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless $self->{replacement};
-	my @number = @{$self}{qw(order preference)};
-	my @string = map { $_->string } @{$self}{qw(flags service regexp replacement)};
-	my @rdata  = @number, @string;
+	return '' unless defined $self->{replacement};
+	my @order = @{$self}{qw(order preference)};
+	my @rdata = ( @order, map $_->string, @{$self}{qw(flags service regexp replacement)} );
 }
 
 
-sub parse_rdata {			## populate RR from rdata in argument list
+sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
-	map { $self->$_(shift) } qw(order preference flags service regexp replacement);
+	foreach (qw(order preference flags service regexp replacement)) { $self->$_(shift) }
 }
 
 
@@ -113,23 +112,15 @@ sub replacement {
 }
 
 
-__PACKAGE__->set_rrsort_func(
-	'order',
-	sub {
-		my ( $a, $b ) = ( $Net::DNS::a, $Net::DNS::b );
-		$a->{order} <=> $b->{order}
-				|| $a->{preference} <=> $b->{preference};
-	} );
+my $function = sub {
+	my ( $a, $b ) = ( $Net::DNS::a, $Net::DNS::b );
+	$a->{order} <=> $b->{order}
+			|| $a->{preference} <=> $b->{preference};
+};
 
-__PACKAGE__->set_rrsort_func(
-	'preference',
-	sub {
-		my ( $a, $b ) = ( $Net::DNS::a, $Net::DNS::b );
-		$a->{preference} <=> $b->{preference}
-				|| $a->{order} <=> $b->{order};
-	} );
+__PACKAGE__->set_rrsort_func( 'order', $function );
 
-__PACKAGE__->set_rrsort_func( 'default_sort', __PACKAGE__->get_rrsort_func('order') );
+__PACKAGE__->set_rrsort_func( 'default_sort', $function );
 
 
 1;
