@@ -1,10 +1,10 @@
 package Net::DNS::RR::NSEC;
 
 #
-# $Id: NSEC.pm 1362 2015-06-23 08:47:14Z willem $
+# $Id: NSEC.pm 1390 2015-09-11 11:42:11Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1362 $)[1];
+$VERSION = (qw$LastChangedRevision: 1390 $)[1];
 
 
 use strict;
@@ -23,7 +23,7 @@ use Net::DNS::DomainName;
 use Net::DNS::Parameters;
 
 
-sub decode_rdata {			## decode rdata from wire-format octet string
+sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 	my ( $data, $offset ) = @_;
 
@@ -33,23 +33,23 @@ sub decode_rdata {			## decode rdata from wire-format octet string
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
-	my $self = shift;
-
-	return '' unless $self->{typebm};
-	join '', $self->{nxtdname}->encode(), $self->{typebm};
-}
-
-
-sub format_rdata {			## format rdata portion of RR string.
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
 	my $nxtdname = $self->{nxtdname} || return '';
-	my @rdata = $nxtdname->string(), $self->typelist;
+	join '', $nxtdname->encode(), $self->{typebm};
 }
 
 
-sub parse_rdata {			## populate RR from rdata in argument list
+sub _format_rdata {			## format rdata portion of RR string.
+	my $self = shift;
+
+	my $nxtdname = $self->{nxtdname} || return '';
+	my @rdata = ( $nxtdname->string(), $self->typelist );
+}
+
+
+sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
 	$self->nxtdname(shift);
@@ -130,7 +130,7 @@ sub _bm2type {
 
 
 sub typebm {				## historical
-	my $self = shift;
+	my $self = shift;					# uncoverable pod
 	$self->{typebm} = shift if scalar @_;
 	return $self->{typebm};
 }
