@@ -1,10 +1,10 @@
 package Net::DNS::RR::OPT;
 
 #
-# $Id: OPT.pm 1367 2015-06-29 08:53:56Z willem $
+# $Id: OPT.pm 1388 2015-09-07 12:28:16Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1367 $)[1];
+$VERSION = (qw$LastChangedRevision: 1388 $)[1];
 
 
 use strict;
@@ -27,7 +27,7 @@ use constant CLASS_TTL_RDLENGTH => length pack 'n N n', (0) x 3;
 use constant OPT => typebyname qw(OPT);
 
 
-sub decode_rdata {			## decode rdata from wire-format octet string
+sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 	my ( $data, $offset ) = @_;
 
@@ -47,7 +47,7 @@ sub decode_rdata {			## decode rdata from wire-format octet string
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
 	my $rdata = '';
@@ -62,7 +62,7 @@ sub encode_rdata {			## encode rdata as wire-format octet string
 sub encode {				## overide RR method
 	my $self = shift;
 
-	my $data = $self->encode_rdata;
+	my $data = $self->_encode_rdata;
 	my $size = $self->size;
 	my @xttl = ( $self->rcode >> 4, $self->version, $self->flags );
 	pack 'C n n C2n n a*', 0, OPT, $size, @xttl, length($data), $data;
@@ -76,9 +76,9 @@ sub string {				## overide RR method
 	my $flags  = sprintf '%04x', $self->flags;
 	my $rcode  = $self->rcode;
 	my $size   = $self->size;
-	my @option = sort $self->options;
-	my @lines  = join "\n;;\t\t",
-			map sprintf( "%s\t%s", ednsoptionbyval($_), unpack 'H*', $self->option($_) ), @option;
+	my @option = sort { $a <=> $b } $self->options;
+	my @lines  = map sprintf( "%s\t%s", ednsoptionbyval($_), unpack 'H*', $self->option($_) ), @option;
+	my @format = join "\n;;\t\t", @lines;
 
 	$rcode = 0 if $rcode < 16;				# weird: 1 .. 15 not EDNS codes!!
 
@@ -91,7 +91,7 @@ sub string {				## overide RR method
 ;;	flags:	$flags
 ;;	rcode:	$rc
 ;;	size:	$size
-;;	option: @lines
+;;	option: @format
 QQ
 }
 
@@ -165,10 +165,10 @@ sub option {
 }
 
 
-sub defined {
+sub _specified {
 	my $self = shift;
-	my @defd = grep $self->{$_}, qw(size flags rcode option);
-	return scalar @defd;
+	my @spec = grep $self->{$_}, qw(size flags rcode option);
+	scalar @spec;
 }
 
 
@@ -238,7 +238,7 @@ header.
 
 16 bit field containing EDNS extended header flags.
 
-=head2 Options
+=head2 options, option
 
 	@option = $packet->edns->options;
 
