@@ -1,10 +1,10 @@
 package Net::DNS::RR::PX;
 
 #
-# $Id: PX.pm 1362 2015-06-23 08:47:14Z willem $
+# $Id: PX.pm 1390 2015-09-11 11:42:11Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1362 $)[1];
+$VERSION = (qw$LastChangedRevision: 1390 $)[1];
 
 
 use strict;
@@ -22,7 +22,7 @@ use integer;
 use Net::DNS::DomainName;
 
 
-sub decode_rdata {			## decode rdata from wire-format octet string
+sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 	my ( $data, $offset, @opaque ) = @_;
 
@@ -32,26 +32,26 @@ sub decode_rdata {			## decode rdata from wire-format octet string
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 	my ( $offset, @opaque ) = @_;
 
-	return '' unless $self->{mapx400};
+	my $mapx400 = $self->{mapx400} || return '';
 	my $rdata = pack( 'n', $self->{preference} );
-	$rdata .= $self->{map822}->encode( $offset + length($rdata), @opaque );
-	$rdata .= $self->{mapx400}->encode( $offset + length($rdata), @opaque );
+	$rdata .= $self->{map822}->encode( $offset + 2, @opaque );
+	$rdata .= $mapx400->encode( $offset + length($rdata), @opaque );
 }
 
 
-sub format_rdata {			## format rdata portion of RR string.
+sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless $self->{mapx400};
-	join ' ', $self->preference, $self->{map822}->string, $self->{mapx400}->string;
+	my $mapx400 = $self->{mapx400} || return '';
+	join ' ', $self->preference, $self->{map822}->string, $mapx400->string;
 }
 
 
-sub parse_rdata {			## populate RR from rdata in argument list
+sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
 	$self->preference(shift);
@@ -84,13 +84,13 @@ sub mapx400 {
 }
 
 
-__PACKAGE__->set_rrsort_func(		## sort RRs in numerically ascending order.
-	'preference',
-	sub { $Net::DNS::a->{'preference'} <=> $Net::DNS::b->{'preference'} }
+my $function = sub {			## sort RRs in numerically ascending order.
+	$Net::DNS::a->{'preference'} <=> $Net::DNS::b->{'preference'};
+};
 
-	);
+__PACKAGE__->set_rrsort_func( 'preference', $function );
 
-__PACKAGE__->set_rrsort_func( 'default_sort', __PACKAGE__->get_rrsort_func('preference') );
+__PACKAGE__->set_rrsort_func( 'default_sort', $function );
 
 
 1;
