@@ -1,10 +1,10 @@
 package Net::DNS::RR::RP;
 
 #
-# $Id: RP.pm 1362 2015-06-23 08:47:14Z willem $
+# $Id: RP.pm 1390 2015-09-11 11:42:11Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1362 $)[1];
+$VERSION = (qw$LastChangedRevision: 1390 $)[1];
 
 
 use strict;
@@ -23,7 +23,7 @@ use Net::DNS::DomainName;
 use Net::DNS::Mailbox;
 
 
-sub decode_rdata {			## decode rdata from wire-format octet string
+sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 	my ( $data, $offset, @opaque ) = @_;
 
@@ -32,25 +32,25 @@ sub decode_rdata {			## decode rdata from wire-format octet string
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 	my ( $offset, @opaque ) = @_;
 
-	return '' unless $self->{txtdname};
+	my $txtdname = $self->{txtdname} || return '';
 	my $rdata = $self->{mbox}->encode( $offset, @opaque );
-	$rdata .= $self->{txtdname}->encode( $offset + length($rdata), @opaque );
+	$rdata .= $txtdname->encode( $offset + length($rdata), @opaque );
 }
 
 
-sub format_rdata {			## format rdata portion of RR string.
+sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless $self->{mbox};
-	join ' ', map $self->{$_}->string, qw(mbox txtdname);
+	my $txtdname = $self->{txtdname} || return '';
+	my @rdata = ( $self->{mbox}->string, $txtdname->string );
 }
 
 
-sub parse_rdata {			## populate RR from rdata in argument list
+sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
 	$self->mbox(shift);
