@@ -1,10 +1,10 @@
 package Net::DNS::RR::SRV;
 
 #
-# $Id: SRV.pm 1362 2015-06-23 08:47:14Z willem $
+# $Id: SRV.pm 1390 2015-09-11 11:42:11Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1362 $)[1];
+$VERSION = (qw$LastChangedRevision: 1390 $)[1];
 
 
 use strict;
@@ -22,7 +22,7 @@ use integer;
 use Net::DNS::DomainName;
 
 
-sub decode_rdata {			## decode rdata from wire-format octet string
+sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 	my ( $data, $offset, @opaque ) = @_;
 
@@ -32,25 +32,25 @@ sub decode_rdata {			## decode rdata from wire-format octet string
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 	my ( $offset, @opaque ) = @_;
 
-	return '' unless $self->{target};
-	my $nums = pack 'n3', $self->priority, $self->weight, $self->port;
-	$nums .= $self->{target}->encode( $offset + length($nums), @opaque );
+	my $target = $self->{target} || return '';
+	my @nums = ( $self->priority, $self->weight, $self->port );
+	pack 'n3 a*', @nums, $target->encode( $offset + 6, @opaque );
 }
 
 
-sub format_rdata {			## format rdata portion of RR string.
+sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless $self->{target};
-	my @rdata = $self->priority, $self->weight, $self->port, $self->{target}->string;
+	my $target = $self->{target} || return '';
+	my @rdata = ( $self->priority, $self->weight, $self->port, $target->string );
 }
 
 
-sub parse_rdata {			## populate RR from rdata in argument list
+sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
 	foreach my $attr (qw(priority weight port target)) {
@@ -91,16 +91,16 @@ sub target {
 }
 
 
-__PACKAGE__->set_rrsort_func(
-	'priority',
-	sub {
-		my ( $a, $b ) = ( $Net::DNS::a, $Net::DNS::b );
-		$a->{priority} <=> $b->{priority}
-				|| $b->{weight} <=> $a->{weight};
-	} );
+# order RRs by numerically increasing priority, decreasing weight
+my $function = sub {
+	my ( $a, $b ) = ( $Net::DNS::a, $Net::DNS::b );
+	$a->{priority} <=> $b->{priority}
+			|| $b->{weight} <=> $a->{weight};
+};
 
+__PACKAGE__->set_rrsort_func( 'priority', $function );
 
-__PACKAGE__->set_rrsort_func( 'default_sort', __PACKAGE__->get_rrsort_func('priority') );
+__PACKAGE__->set_rrsort_func( 'default_sort', $function );
 
 
 1;
