@@ -1,10 +1,10 @@
 package Net::DNS::Resolver;
 
 #
-# $Id: Resolver.pm 1356 2015-06-11 08:44:12Z willem $
+# $Id: Resolver.pm 1399 2015-09-15 18:50:43Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1356 $)[1];
+$VERSION = (qw$LastChangedRevision: 1399 $)[1];
 
 =head1 NAME
 
@@ -17,17 +17,18 @@ use strict;
 
 use vars qw(@ISA);
 
-BEGIN {
-	for ( $^O, 'UNIX' ) {
-		my $class = join '::', __PACKAGE__, $_;
-		return @ISA = ($class) if eval "require $class;";
-	}
+
+local $SIG{__DIE__};
+local $SIG{__WARN__} = sub { };
+
+for ( $^O, 'UNIX' ) {
+	my @parent = ( join '::', __PACKAGE__, $_ );
+	@ISA = @parent if scalar eval "require @parent";
+	last if @ISA;
 }
 
-
 die 'failed to load platform specific resolver component' unless @ISA;
 
-
 1;
 
 __END__
