@@ -1,10 +1,10 @@
 package Net::DNS::Resolver::MSWin32;
 
 #
-# $Id: MSWin32.pm 1370 2015-07-01 13:48:40Z willem $
+# $Id: MSWin32.pm 1381 2015-08-25 07:36:09Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1370 $)[1];
+$VERSION = (qw$LastChangedRevision: 1381 $)[1];
 
 =head1 NAME
 
@@ -31,7 +31,9 @@ BEGIN {
 }
 
 
-sub _untaint { map defined && /^(.+)$/ ? $1 : (), @_; }
+sub _untaint {
+	map { m/^(.*)$/; $1 } grep defined, @_;
+}
 
 
 sub init {
