@@ -1,10 +1,10 @@
 package Net::DNS::Resolver::Recurse;
 
 #
-# $Id: Recurse.pm 1339 2015-04-16 13:01:17Z willem $
+# $Id: Recurse.pm 1390 2015-09-11 11:42:11Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1339 $)[1];
+$VERSION = (qw$LastChangedRevision: 1390 $)[1];
 
 
 =head1 NAME
@@ -42,8 +42,8 @@ Additional module-specific methods are described below.
 
 =head2 hints
 
-This method specifies a list of the IP addresses used to locate
-the authoritative name servers for the root (.) zone.
+This method specifies a list of the IP addresses of nameservers to
+be used to discover the addresses of the root nameservers.
 
     $resolver->hints(@ip);
 
@@ -61,8 +61,9 @@ my $root;
 sub hints {
 	my $self = shift;
 
-	@hints = @_ if scalar @_;
-	return @hints;
+	return @hints unless scalar @_;
+	$root  = undef;
+	@hints = @_;
 }
 
 
@@ -92,65 +93,64 @@ sub send {
 	$original = $question unless ref($original);		# to preserve original request
 
 	my ( $head, @tail ) = $question->{qname}->label;
-	unless ($head) {
+	unless ( defined $head ) {
 		return $root if $root;				# root servers cached indefinitely
 
-		my $defres = new Net::DNS::Resolver();
-		$defres->nameservers( $res->hints ) || $defres->nameservers( $res->_hints );
+		my $defres = new Net::DNS::Resolver( retry => 1 );
+		my @config = $defres->nameservers( $res->hints );
+		$defres->nameservers( @config, $res->_hints );	# hints, resolv.conf, _hints
+		$defres->udppacketsize(1280);
 
-		my $packet = $defres->send( '.', 'NS' );	# specified hint server
-		$res->{callback}->($packet) if $res->{callback};
-		my @auth = grep $_->type eq 'NS', $packet->answer, $packet->authority;
-		my %auth = map { lc $_->nsdname => 1 } @auth;
-		my @glue = grep $auth{lc $_->name}, $packet->additional;
+		my $packet = $defres->send( '.', 'NS' );
 		my %glue;
-		foreach ( grep $_->type eq 'A',	   @glue ) { push @{$glue{lc $_->name}}, $_->address }
-		foreach ( grep $_->type eq 'AAAA', @glue ) { push @{$glue{lc $_->name}}, $_->address }
-		my @ip = map @$_, values %glue;
-		return $root = $packet if @ip && $packet->header->aa;
 
-		$defres->nameservers(@ip);
-		$defres->recurse(0);
-		foreach my $ns ( map $_->nsdname, @auth ) {
-			$defres->nameservers($ns) unless @ip;
-			$packet = $defres->send( '.', 'NS' );	# authoritative root server
-			$res->{callback}->($packet) if $res->{callback};
-			my @auth = grep $_->type eq 'NS', $packet->answer, $packet->authority;
+		# uncoverable branch false	# depends on external data
+		if ($packet) {
+			my @rr = $packet->answer, $packet->authority;
+			my @auth = grep $_->type eq 'NS', @rr;
 			my %auth = map { lc $_->nsdname => 1 } @auth;
 			my @glue = grep $auth{lc $_->name}, $packet->additional;
-			my @ip = grep $_->type eq 'A', @glue;
-			push @ip, grep $_->type eq 'AAAA', @glue;
-			return $root = $packet if @ip && @auth;
+			foreach ( grep $_->can('address'), @glue ) {
+				push @{$glue{lc $_->name}}, $_->address;
+			}
 		}
-		return $packet;
+
+		$defres->nameservers( $res->_hints );
+		$defres->nameservers( map @$_, values %glue );
+		$defres->recurse(0);
+		return $root = $defres->send( '.', 'NS' );	# authoritative server
 	}
 
 	my $domain = lc join( '.', @tail ) || '.';
 	my $nslist = $res->{cache}->{$domain} ||= [];
 	if ( scalar @$nslist ) {
-		print ";; using cached nameservers for $domain\n" if $res->{debug};
+		$self->_diag(";; using cached nameservers for $domain\n");
 	} else {
 		$domain = lc $question->qname if $question->qtype ne 'NULL';
-		my $packet = $res->send( $domain, 'NULL', 'ANY', $original ) || return;
-		return $packet unless $packet->header->rcode eq 'NOERROR';
+		my $packet = $res->send( $domain, 'NULL', 'IN', $original );
+
+		# uncoverable branch true	# depends on external data
+		return unless $packet;
 
 		my @answer = $packet->answer;			# return authoritative answer
 		return $packet if $packet->header->aa && grep $_->name eq $original->qname, @answer;
 
 		my @auth = grep $_->type eq 'NS', $packet->answer, $packet->authority;
 		my %auth = map { lc $_->nsdname => lc $_->name } @auth;
-		my @glue = grep $auth{lc $_->name}, $packet->additional;
 		my %glue;
-		foreach ( grep $_->type eq 'A',	   @glue ) { push @{$glue{lc $_->name}}, $_->address }
-		foreach ( grep $_->type eq 'AAAA', @glue ) { push @{$glue{lc $_->name}}, $_->address }
+		my @glue = grep $auth{lc $_->name}, $packet->additional;
+		foreach ( grep $_->can('address'), @glue ) {
+			push @{$glue{lc $_->name}}, $_->address;
+		}
 
 		my %zone = reverse %auth;
 		foreach my $zone ( keys %zone ) {
-			print ";; cache nameservers for $zone\n" if $res->{debug};
+			$self->_diag(";; cache nameservers for $zone\n");
 			my @nsname = grep $auth{$_} eq $zone, keys %auth;
-			$nslist = $res->{cache}->{$zone} ||= [];
-			@$nslist = map $glue{$_} || $_, @nsname;
-			last if $zone eq $domain;
+			my @list = map $glue{$_} ? $glue{$_} : $_, @nsname;
+			@{$res->{cache}->{$zone}} = @list;
+			return $packet if length($zone) > length($domain);
+			push @$nslist, @list;
 		}
 	}
 
@@ -162,30 +162,45 @@ sub send {
 	splice @a, 0, 0, splice( @a, int( rand scalar @a ) );	# cut deck
 
 	foreach (@a) {
-		$res->empty_nameservers();
-		$res->nameservers( map @$_, @a );
-		my $reply = $res->send($query) || last;
-		$res->{callback}->($reply) if $res->{callback};
+		my @ns = $res->nameservers(@$_);
+
+		# uncoverable branch true	# depends on external data
+		next unless scalar @ns;
+
+		my $reply = $res->send($query);
+
+		# uncoverable branch true	# depends on external data
+		next unless $reply;
+
+		# uncoverable branch true	# depends on external data
+		next unless $reply->header->rcode eq 'NOERROR';
+
+		$self->_callback($reply);
 		return $reply;
 	}
 
-	foreach my $ns (@$nslist) {
-		next if ref($ns);
-		my $name = $ns;
-		print ";; find missing glue for $name\n" if $res->{debug};
-		$ns = [];					# substitute IP list in situ
-		$res->empty_nameservers();
-		@$ns = $res->nameservers($name);
+	foreach my $ns ( grep !ref($_), @$nslist ) {
+		$self->_diag(";; find missing glue for $ns\n");
+		$ns = [$res->nameservers($ns)];			# substitute IP list in situ
+
+		# uncoverable branch true	# depends on external data
 		next unless scalar @$ns;
-		my $reply = $res->send($query) || next;
-		$res->{callback}->($reply) if $res->{callback};
+
+		my $reply = $res->send($query);
+
+		# uncoverable branch true	# depends on external data
+		next unless $reply;
+
+		# uncoverable branch true	# depends on external data
+		next unless $reply->header->rcode eq 'NOERROR';
+
+		$self->_callback($reply);
 		return $reply;
 	}
-	return;
 }
 
 
-sub query_dorecursion { &send; }	## historical
+sub query_dorecursion { &send; }				# uncoverable pod
 
 
 =head2 callback
@@ -198,8 +213,6 @@ For example to emulate dig's C<+trace> function:
     my $coderef = sub {
 	my $packet = shift;
 
-	$_->print for $packet->additional;
-
 	printf ";; Received %d bytes from %s\n\n",
 		$packet->answersize, $packet->answerfrom;
     };
@@ -212,25 +225,28 @@ for queries for missing glue records.
 =cut
 
 sub callback {
-	my ( $self, $sub ) = @_;
-
-	$self->{callback} = $sub if defined $sub && ref($sub) eq 'CODE';
-	return $self->{callback};
-}
-
-sub recursion_callback { &callback; }	## historical
-
-
-sub bgsend {
 	my $self = shift;
-	my $class = ref($self) || $self;
-	Carp::croak "method ${class}::bgsend undefined";
+
+	( $self->{callback} ) = grep ref($_) eq 'CODE', @_;
 }
 
+sub _callback {
+	my $callback = shift->{callback};
+	$callback->(@_) if $callback;
+}
+
+sub recursion_callback { &callback; }				# uncoverable pod
+
 
 ########################################
 
-sub _hints {				## default hints
+sub _diag {				## debug output
+	my $self = shift;
+	print @_ if $self->{debug};				# uncoverable branch true
+}
+
+
+{
 	require Net::DNS::ZoneFile;
 
 	my $dug = new Net::DNS::ZoneFile( \*DATA );
@@ -238,11 +254,15 @@ sub _hints {				## default hints
 
 	my @auth = grep $_->type eq 'NS', @rr;
 	my %auth = map { lc $_->nsdname => 1 } @auth;
-	my @glue = grep $auth{lc $_->name}, @rr;
 	my %glue;
-	foreach ( grep $_->type eq 'A',	   @glue ) { push @{$glue{lc $_->name}}, $_->address }
-	foreach ( grep $_->type eq 'AAAA', @glue ) { push @{$glue{lc $_->name}}, $_->address }
+	my @glue = grep $auth{lc $_->name}, @rr;
+	foreach ( grep $_->can('address'), @glue ) {
+		push @{$glue{lc $_->name}}, $_->address;
+	}
 	my @ip = map @$_, values %glue;
+
+
+	sub _hints { @ip; }		## default hints
 }
 
 
