@@ -1,10 +1,10 @@
 package Net::DNS::Resolver::cygwin;
 
 #
-# $Id: cygwin.pm 1348 2015-05-27 09:55:13Z willem $
+# $Id: cygwin.pm 1381 2015-08-25 07:36:09Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1348 $)[1];
+$VERSION = (qw$LastChangedRevision: 1381 $)[1];
 
 =head1 NAME
 
@@ -30,7 +30,9 @@ sub getregkey {
 }
 
 
-sub _untaint { map defined && /^(.+)$/ ? $1 : (), @_; }
+sub _untaint {
+	map { m/^(.*)$/; $1 } grep defined, @_;
+}
 
 
 sub init {
