@@ -1,10 +1,10 @@
 package Net::DNS::Text;
 
 #
-# $Id: Text.pm 1370 2015-07-01 13:48:40Z willem $
+# $Id: Text.pm 1381 2015-08-25 07:36:09Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1370 $)[1];
+$VERSION = (qw$LastChangedRevision: 1381 $)[1];
 
 
 =head1 NAME
@@ -150,7 +150,14 @@ sub encode {
 }
 
 
-# Returns unbounded wire-format text
+=head2 raw
+
+    $data = $object->raw;
+
+Returns the wire-format encoded representation of the text object
+without the explicit length field.
+
+=cut
 
 sub raw {
 	my $self = shift;
