@@ -1,9 +1,10 @@
 package Net::DNS::ZoneFile;
 
-use strict;
-use warnings;
-
-our $VERSION = (qw$Id: ZoneFile.pm 2000 2025-01-02 16:34:13Z willem $)[2];
+#
+# $Id: ZoneFile.pm 1370 2015-07-01 13:48:40Z willem $
+#
+use vars qw($VERSION);
+$VERSION = (qw$LastChangedRevision: 1370 $)[1];
 
 
 =head1 NAME
@@ -12,15 +13,15 @@ Net::DNS::ZoneFile - DNS zone file
 
 =head1 SYNOPSIS
 
- use Net::DNS::ZoneFile;
+    use Net::DNS::ZoneFile;
 
- $zonefile = Net::DNS::ZoneFile->new( 'named.example' );
+    $zonefile = new Net::DNS::ZoneFile( 'named.example' );
 
- while ( $rr = $zonefile->read ) {
-		$rr->print;
- }
+    while ( $rr = $zonefile->read ) {
+	$rr->print;
+    }
 
- @zone = $zonefile->read;
+    @zone = $zonefile->read;
 
 
 =head1 DESCRIPTION
@@ -42,21 +43,15 @@ automatically to all subsequent records.
 =cut
 
 
+use strict;
 use integer;
 use Carp;
 
-use base qw(Exporter);
-our @EXPORT = qw(parse read readfh);
+require FileHandle;
 
-use constant UTF8 => scalar eval {	## not UTF-EBCDIC  [see Unicode TR#16 3.6]
-	require Encode;
-	Encode::encode_utf8( chr(182) ) eq pack( 'H*', 'C2B6' );
-};
+use Net::DNS;
 
-require IO::File;
-require PerlIO;
-require Net::DNS::Domain;
-require Net::DNS::RR;
+use constant PERLIO => defined eval { require PerlIO; };
 
 
 =head1 METHODS
@@ -64,10 +59,10 @@ require Net::DNS::RR;
 
 =head2 new
 
- $zonefile = Net::DNS::ZoneFile->new( 'filename', ['example.com'] );
+    $zonefile = new Net::DNS::ZoneFile( 'filename', ['example.com'] );
 
- $handle   = IO::File->new( 'filename', '<:encoding(ISO8859-7)' );
- $zonefile = Net::DNS::ZoneFile->new( $handle, ['example.com'] );
+    $handle   = new FileHandle( 'filename', '<:encoding(ISO8859-7)' );
+    $zonefile = new Net::DNS::ZoneFile( $handle, ['example.com'] );
 
 The new() constructor returns a Net::DNS::ZoneFile object which
 represents the zone file specified in the argument list.
@@ -77,40 +72,34 @@ exhausted or all references to the ZoneFile object cease to exist.
 
 The optional second argument specifies $ORIGIN for the zone file.
 
-Zone files are presumed to be UTF-8 encoded where that is supported.
-
-Alternative character encodings may be specified indirectly by creating
-a file handle with the desired encoding layer, which is then passed as
-an argument to new(). The specified encoding is propagated to files
-introduced by $INCLUDE directives.
+Character encoding is specified indirectly by creating a FileHandle
+with the desired encoding layer, which is then passed as an argument
+to new(). The specified encoding is propagated to files introduced
+by $include directives.
 
 =cut
 
 sub new {
-	my ( $class, $filename, $origin ) = @_;
-	my $self = bless {fileopen => {}}, $class;
+	my $self = bless {}, shift;
+	my $file = shift;
+	$self->_origin(shift);
 
-	$self->_origin($origin);
-
-	if ( ref($filename) ) {
-		$self->{filehandle} = $self->{filename} = $filename;
-		return $self if ref($filename) =~ /IO::File|FileHandle|GLOB|Text/;
+	if ( ref($file) ) {
+		$self->{filename} = $self->{handle} = $file;
+		return $self if ref($file) =~ /FileHandle|IO::File|GLOB|Text/;
 		croak 'argument not a file handle';
 	}
 
-	croak 'filename argument undefined' unless $filename;
-	my $discipline = UTF8 ? '<:encoding(UTF-8)' : '<';
-	$self->{filehandle} = IO::File->new( $filename, $discipline ) or croak "$filename: $!";
-	$self->{fileopen}->{$filename}++;
-	$self->{filename} = $filename;
+	$self->{filename} = $file ||= '';
+	$self->{handle} = new FileHandle($file) or croak qq($! "$file");
 	return $self;
 }
 
 
 =head2 read
 
- $rr = $zonefile->read;
- @rr = $zonefile->read;
+    $rr = $zonefile->read;
+    @rr = $zonefile->read;
 
 When invoked in scalar context, read() returns a Net::DNS::RR object
 representing the next resource record encountered in the zone file,
@@ -134,19 +123,14 @@ sub read {
 	if (wantarray) {
 		my @zone;					# return entire zone
 		eval {
-			local $SIG{__DIE__};
-			while ( my $rr = $self->_getRR ) {
-				push( @zone, $rr );
-			}
+			my $rr;
+			push( @zone, $rr ) while $rr = $self->_getRR;
 		};
 		croak join ' ', $@, ' file', $self->name, 'line', $self->line, "\n " if $@;
 		return @zone;
 	}
 
-	my $rr = eval {
-		local $SIG{__DIE__};
-		$self->_getRR;					# return single RR
-	};
+	my $rr = eval { $self->_getRR };			# return single RR
 	croak join ' ', $@, ' file', $self->name, 'line', $self->line, "\n " if $@;
 	return $rr;
 }
@@ -154,11 +138,11 @@ sub read {
 
 =head2 name
 
- $filename = $zonefile->name;
+    $filename = $zonefile->name;
 
-Returns the name of the current zone file.
-Embedded $INCLUDE directives will cause this to differ from the
-filename argument supplied when the object was created.
+Returns the name of the zone file from which RRs will be read.
+$INCLUDE directives will cause this to differ from the filename
+argument supplied when the object was created.
 
 =cut
 
@@ -169,7 +153,7 @@ sub name {
 
 =head2 line
 
- $line = $zonefile->line;
+    $line = $zonefile->line;
 
 Returns the number of the last line read from the current zone file.
 
@@ -178,13 +162,13 @@ Returns the number of the last line read from the current zone file.
 sub line {
 	my $self = shift;
 	return $self->{eom} if defined $self->{eom};
-	return $self->{filehandle}->input_line_number;
+	return $self->{handle}->input_line_number;
 }
 
 
 =head2 origin
 
- $origin = $zonefile->origin;
+    $origin = $zonefile->origin;
 
 Returns the fully qualified name of the current origin within the
 zone file.
@@ -193,13 +177,13 @@ zone file.
 
 sub origin {
 	my $context = shift->{context};
-	return &$context( sub { Net::DNS::Domain->new('@') } )->string;
+	return &$context( sub { new Net::DNS::Domain('@') } )->string;
 }
 
 
 =head2 ttl
 
- $ttl = $zonefile->ttl;
+    $ttl = $zonefile->ttl;
 
 Returns the default TTL as specified by the $TTL directive.
 
@@ -215,22 +199,17 @@ sub ttl {
 Applications which depended on the defunct Net::DNS::ZoneFile 1.04
 CPAN distribution will continue to operate with minimal change using
 the compatibility interface described below.
-New application code should use the object-oriented interface.
 
- use Net::DNS::ZoneFile;
+    use Net::DNS::ZoneFile;
 
- $listref = Net::DNS::ZoneFile->read( $filename );
- $listref = Net::DNS::ZoneFile->read( $filename, $include_dir );
+    $listref = Net::DNS::ZoneFile->read( $filename, $include_dir );
 
- $listref = Net::DNS::ZoneFile->readfh( $filehandle );
- $listref = Net::DNS::ZoneFile->readfh( $filehandle, $include_dir );
+    $listref = Net::DNS::ZoneFile->readfh( $handle, $include_dir );
 
- $listref = Net::DNS::ZoneFile->parse( $string );
- $listref = Net::DNS::ZoneFile->parse( $string, $include_dir );
- $listref = Net::DNS::ZoneFile->parse( \$string );
- $listref = Net::DNS::ZoneFile->parse( \$string, $include_dir );
+    $listref = Net::DNS::ZoneFile->parse(  $string, $include_dir );
+    $listref = Net::DNS::ZoneFile->parse( \$string, $include_dir );
 
- $_->print for @$listref;
+    $_->print for @$listref;
 
 The optional second argument specifies the default path for filenames.
 The current working directory is used by default.
@@ -238,23 +217,24 @@ The current working directory is used by default.
 Although not available in the original implementation, the RR list can
 be obtained directly by calling any of these methods in list context.
 
- @rr = Net::DNS::ZoneFile->read( $filename, $include_dir );
-
-The partial result is returned if an error is encountered by the parser.
+    @rr = Net::DNS::ZoneFile->read( $filename, $include_dir );
 
 
 =head2 read
 
- $listref = Net::DNS::ZoneFile->read( $filename );
- $listref = Net::DNS::ZoneFile->read( $filename, $include_dir );
+    $listref = Net::DNS::ZoneFile->read( $filename, $include_dir );
+    @rr = Net::DNS::ZoneFile->read( $filename, $include_dir );
 
-read() parses the contents of the specified file
-and returns a reference to the list of Net::DNS::RR objects.
-The return value is undefined if an error is encountered by the parser.
+read() parses the specified zone file and returns a reference to the
+list of Net::DNS::RR objects representing the RRs in the file.
+The return value is undefined if the zone data can not be parsed.
+
+When called in list context, the partial result is returned if an
+error is encountered by the parser.
 
 =cut
 
-our $include_dir;			## dynamically scoped
+use vars qw($include_dir);		## dynamically scoped
 
 sub _filename {				## rebase unqualified filename
 	my $name = shift;
@@ -262,7 +242,6 @@ sub _filename {				## rebase unqualified filename
 	return $name unless $include_dir;
 	require File::Spec;
 	return $name if File::Spec->file_name_is_absolute($name);
-	return $name if -f $name;	## file in current directory
 	return File::Spec->catfile( $include_dir, $name );
 }
 
@@ -273,10 +252,9 @@ sub _read {
 	my $filename = shift;
 	local $include_dir = shift;
 
-	my $zonefile = Net::DNS::ZoneFile->new( _filename($filename) );
+	my $zonefile = new Net::DNS::ZoneFile( _filename($filename) );
 	my @zone;
 	eval {
-		local $SIG{__DIE__};
 		my $rr;
 		push( @zone, $rr ) while $rr = $zonefile->_getRR;
 	};
@@ -288,15 +266,24 @@ sub _read {
 
 {
 
-	package Net::DNS::ZoneFile::Text;	## no critic ProhibitMultiplePackages
+	package Net::DNS::ZoneFile::Text;
 
 	use overload ( '<>' => 'readline' );
 
+	use constant OVERLOAD_OK => scalar eval { no integer; $] > 5.006; };
+
 	sub new {
-		my ( $class, $data ) = @_;
-		my $self = bless {}, $class;
+		my $self = bless {}, shift;
+		my $data = shift;
 		$self->{data} = [split /\n/, ref($data) ? $$data : $data];
-		return $self;
+
+		return OVERLOAD_OK ? $self : do {		# Plan B
+			require IO::File;
+			my $fh = IO::File->new_tmpfile() or die "$!";
+			while ( my $line = $self->readline ) { print $fh $line, "\n"; }
+			seek $fh, 0, 0;
+			return $fh;
+		};
 	}
 
 	sub readline {
@@ -319,12 +306,11 @@ sub _read {
 
 =head2 readfh
 
- $listref = Net::DNS::ZoneFile->readfh( $filehandle );
- $listref = Net::DNS::ZoneFile->readfh( $filehandle, $include_dir );
+    $listref = Net::DNS::ZoneFile->readfh( $handle, $include_dir );
 
-readfh() parses data from the specified file handle
-and returns a reference to the list of Net::DNS::RR objects.
-The return value is undefined if an error is encountered by the parser.
+readfh() parses data from the specified file handle and returns a
+reference to the list of Net::DNS::RR objects representing the RRs
+in the file.
 
 =cut
 
@@ -335,23 +321,18 @@ sub readfh {
 
 =head2 parse
 
- $listref = Net::DNS::ZoneFile->parse( $string );
- $listref = Net::DNS::ZoneFile->parse( $string, $include_dir );
- $listref = Net::DNS::ZoneFile->parse( \$string );
- $listref = Net::DNS::ZoneFile->parse( \$string, $include_dir );
+    $listref = Net::DNS::ZoneFile->parse(  $string, $include_dir );
+    $listref = Net::DNS::ZoneFile->parse( \$string, $include_dir );
 
-parse() interprets the text in the argument string
-and returns a reference to the list of Net::DNS::RR objects.
-The return value is undefined if an error is encountered by the parser.
+parse() interprets the zone file text in the argument string and
+returns a reference to the list of Net::DNS::RR objects representing
+the RRs.
 
 =cut
 
 sub parse {
-	my ($arg1) = @_;
-	shift if $arg1 eq __PACKAGE__;
-	my $string  = shift;
-	my @include = grep {defined} shift;
-	return &readfh( Net::DNS::ZoneFile::Text->new($string), @include );
+	my ($text) = reverse @_;
+	return &readfh( new Net::DNS::ZoneFile::Text($text), @_ );
 }
 
 
@@ -360,34 +341,41 @@ sub parse {
 
 {
 
-	package Net::DNS::ZoneFile::Generator;	## no critic ProhibitMultiplePackages
+	package Net::DNS::ZoneFile::Generator;
 
 	use overload ( '<>' => 'readline' );
 
+	use constant OVERLOAD_OK => scalar eval { no integer; $] > 5.006; };
+
 	sub new {
-		my ( $class, $range, $template, $line ) = @_;
-		my $self = bless {}, $class;
+		my $self = bless {}, shift;
+		my ( $range, $template, $line ) = @_;
+
+		$template =~ s/\\\$/\\036/g;			# disguise escaped dollar
+		$template =~ s/\$\$/\\036/g;			# disguise escaped dollar
 
 		my ( $bound, $step ) = split m#[/]#, $range;	# initial iterator state
 		my ( $first, $last ) = split m#[-]#, $bound;
 		$first ||= 0;
 		$last  ||= $first;
-		$step  ||= 1;					# coerce step to match range
-		$step = ( $last < $first ) ? -abs($step) : abs($step);
+		$step = abs( $step || 1 );			# coerce step to match range
+		$step = -$step if $last < $first;
 		$self->{count} = int( ( $last - $first ) / $step ) + 1;
 
-		for ($template) {
-			s/\\\$/\\036/g;				# disguise escaped dollar
-			s/\$\$/\\036/g;				# disguise escaped dollar
-			s/^"(.*)"$/$1/s;			# unwrap BIND's quoted template
-			@{$self}{qw(instant step template line)} = ( $first, $step, $_, $line );
-		}
-		return $self;
+		@{$self}{qw(instant step template line)} = ( $first, $step, $template, $line );
+
+		return OVERLOAD_OK ? $self : do {		# Plan B
+			require IO::File;
+			my $fh = IO::File->new_tmpfile() or die "$!";
+			while ( my $line = $self->readline ) { print $fh $line, "\n"; }
+			seek $fh, 0, 0;
+			return $fh;
+		};
 	}
 
 	sub readline {
 		my $self = shift;
-		return unless $self->{count}-- > 0;		# EOF
+		return undef unless $self->{count}-- > 0;	# EOF
 
 		my $instant = $self->{instant};			# update iterator state
 		$self->{instant} += $self->{step};
@@ -399,7 +387,6 @@ sub parse {
 		}
 
 		s/\$/$instant/eg;				# interpolate $
-		s/\\036/\$/g;					# reinstate escaped $
 		return $_;
 	}
 
@@ -424,8 +411,8 @@ sub parse {
 		return substr sprintf( "%01.$digit$format", $value ), -$length if $format =~ /[doxX]/;
 
 		my $nibble = join( '.', split //, sprintf ".%32.32lx", $value );
-		return reverse lc( substr $nibble, -$length ) if $format =~ /[n]/;
-		return reverse uc( substr $nibble, -$length ) if $format =~ /[N]/;
+		return lc reverse substr $nibble, -$length if $format =~ /[n]/;
+		return uc reverse substr $nibble, -$length if $format =~ /[N]/;
 		die "unknown $format format";
 	}
 
@@ -435,92 +422,81 @@ sub parse {
 sub _generate {				## expand $GENERATE into input stream
 	my ( $self, $range, $template ) = @_;
 
-	my $handle = Net::DNS::ZoneFile::Generator->new( $range, $template, $self->line );
+	my $handle = new Net::DNS::ZoneFile::Generator( $range, $template, $self->line );
 
+	delete $self->{latest};					# forbid empty owner field
 	$self->{parent} = bless {%$self}, ref($self);		# save state, create link
-	delete $self->{latest};					# forget current domain name
-	return $self->{filehandle} = $handle;
+	$self->{handle} = $handle;
 }
 
 
-my $LEX_REGEX = q/("[^"]*"|"[^"]*$)|;[^\n]*|([()])|[ \t\n\r\f]+/;
+my $LEX_REGEX = q/("[^"]*"|"[^"]*$)|;[^\n]*|([()])|(^\s)|[ \t\n\r\f]/;
 
 sub _getline {				## get line from current source
 	my $self = shift;
 
-	my $fh = $self->{filehandle};
+	my $fh = $self->{handle};
 	while (<$fh>) {
 		next if /^\s*;/;				# discard comment line
 		next unless /\S/;				# discard blank line
 
-		if (/["(]/) {
+		if (/[(]/) {					# concatenate multi-line RR
 			s/\\\\/\\092/g;				# disguise escaped escape
 			s/\\"/\\034/g;				# disguise escaped quote
 			s/\\\(/\\040/g;				# disguise escaped bracket
 			s/\\\)/\\041/g;				# disguise escaped bracket
 			s/\\;/\\059/g;				# disguise escaped semicolon
-			my @token = grep { defined && length } split /(^\s)|$LEX_REGEX/o;
-
-			while ( $token[-1] =~ /^"[^"]*$/ ) {	# multiline quoted string
-				$_ = pop(@token) . <$fh>;	# reparse fragments
-				s/\\\\/\\092/g;			# disguise escaped escape
-				s/\\"/\\034/g;			# disguise escaped quote
-				s/\\\(/\\040/g;			# disguise escaped bracket
-				s/\\\)/\\041/g;			# disguise escaped bracket
-				s/\\;/\\059/g;			# disguise escaped semicolon
-				push @token, grep { defined && length } split /$LEX_REGEX/o;
-				$_ = join ' ', @token;		# reconstitute RR string
-			}
-
-			if ( grep { $_ eq '(' } @token ) {	# concatenate multiline RR
-				until ( grep { $_ eq ')' } @token ) {
-					$_ = pop(@token) . <$fh>;
+			my @token = grep defined && length, split /$LEX_REGEX/o;
+			if ( grep( $_ eq '(', @token ) && !grep( $_ eq ')', @token ) ) {
+				while (<$fh>) {
+					$_ = pop(@token) . $_;	# splice fragmented string
 					s/\\\\/\\092/g;		# disguise escaped escape
 					s/\\"/\\034/g;		# disguise escaped quote
 					s/\\\(/\\040/g;		# disguise escaped bracket
 					s/\\\)/\\041/g;		# disguise escaped bracket
 					s/\\;/\\059/g;		# disguise escaped semicolon
-					push @token, grep { defined && length } split /$LEX_REGEX/o;
-					chomp $token[-1] unless $token[-1] =~ /^"[^"]*$/;
+					my @part = grep defined && length, split /$LEX_REGEX/o;
+					push @token, @part;
+					last if grep $_ eq ')', @part;
 				}
 				$_ = join ' ', @token;		# reconstitute RR string
 			}
 		}
 
-		return $_ unless /^[\$]/;			# RR string
+		return $_ unless /^\$/;				# RR string
 
-		my @token = grep { defined && length } split /$LEX_REGEX/o;
 		if (/^\$INCLUDE/) {				# directive
-			my ( $keyword, @argument ) = @token;
+			my ( $keyword, @argument ) = split;
 			die '$INCLUDE incomplete' unless @argument;
 			$fh = $self->_include(@argument);
 
 		} elsif (/^\$GENERATE/) {			# directive
-			my ( $keyword, $range, @template ) = @token;
-			die '$GENERATE incomplete' unless @template;
-			$fh = $self->_generate( $range, "@template" );
+			my ( $keyword, $range, @template ) = split;
+			die '$GENERATE incomplete' unless $range;
+			$fh = $self->_generate( $range, "@template\n" );
 
 		} elsif (/^\$ORIGIN/) {				# directive
-			my ( $keyword, $origin ) = @token;
-			die '$ORIGIN incomplete' unless defined $origin;
-			$self->_origin($origin);
+			my ( $keyword, $origin, @etc ) = split;
+			die '$ORIGIN incomplete' unless $origin;
+			my $context = $self->{context};
+			&$context( sub { $self->_origin($origin); } );
 
 		} elsif (/^\$TTL/) {				# directive
-			my ( $keyword, $ttl ) = @token;
+			my ( $keyword, $ttl, @etc ) = split;
 			die '$TTL incomplete' unless defined $ttl;
-			$self->{TTL} = Net::DNS::RR::ttl( {}, $ttl );
+			$self->{TTL} = new Net::DNS::RR(". $ttl IN A")->ttl;
 
 		} else {					# unrecognised
-			my ($keyword) = @token;
-			die qq[unknown "$keyword" directive];
+			my ($keyword) = split;
+			die "unknown '$keyword' directive";
 		}
 	}
 
 	$self->{eom} = $self->line;				# end of file
 	$fh->close();
-	my $link = $self->{parent} || return;			# end of zone
+	my $link = $self->{parent} || return undef;		# end of zone
 	%$self = %$link;					# end $INCLUDE
-	return $self->_getline;					# resume input
+	$self->_getline;					# resume input
 }
 
 
@@ -528,53 +504,46 @@ sub _getRR {				## get RR from current source
 	my $self = shift;
 
 	local $_;
-	$self->_getline || return;				# line already in $_
+	$self->_getline || return undef;			# line already in $_
 
 	my $noname = s/^\s/\@\t/;				# placeholder for empty RR name
 
 	# construct RR object with context specific dynamically scoped $ORIGIN
 	my $context = $self->{context};
-	my $rr	    = &$context( sub { Net::DNS::RR->_new_string($_) } );
+	my $rr = &$context( sub { Net::DNS::RR->_new_string($_) } );
 
-	my $latest = $self->{latest};				# overwrite placeholder
-	$rr->{owner} = $latest->{owner} if $noname && $latest;
+	$rr->{owner} = $self->{latest}->{owner} if $noname;	# overwrite placeholder
 
 	$self->{class} = $rr->class unless $self->{class};	# propagate RR class
 	$rr->class( $self->{class} );
 
-	unless ( defined $self->{TTL} ) {
-		$self->{TTL} = $rr->minimum if $rr->type eq 'SOA';    # default TTL
-	}
-	$rr->{ttl} = $self->{TTL} unless defined $rr->{ttl};
+	$self->{TTL} ||= $rr->minimum if $rr->type eq 'SOA';	# default TTL
+	$rr->{'ttl'} = $self->{TTL} unless defined $rr->{'ttl'};
 
 	return $self->{latest} = $rr;
 }
 
 
 sub _include {				## open $INCLUDE file
-	my ( $self, $include, $origin ) = @_;
+	my $self = shift;
+	my $file = _filename(shift);
+	my $root = shift;
 
-	my $filename = _filename($include);
-	die qq(\$INCLUDE $filename: Unexpected recursion) if $self->{fileopen}->{$filename}++;
-
-	my $discipline = join( ':', '<', PerlIO::get_layers $self->{filehandle} );
-	my $filehandle = IO::File->new( $filename, $discipline ) or die qq(\$INCLUDE $filename: $!);
+	my @discipline = PERLIO ? ( join ':', '<', PerlIO::get_layers $self->{handle} ) : ();
+	my $handle = new FileHandle( $file, @discipline ) or croak qq($! "$file");
 
+	delete $self->{latest};					# forbid empty owner field
 	$self->{parent} = bless {%$self}, ref($self);		# save state, create link
-	delete $self->{latest};					# forget current domain name
-	$self->_origin($origin) if $origin;
-	$self->{filename} = $filename;
-	return $self->{filehandle} = $filehandle;
+	$self->{context} = origin Net::DNS::Domain($root) if $root;
+	$self->{filename} = $file;
+	return $self->{handle} = $handle;
 }
 
 
 sub _origin {				## change $ORIGIN (scope: current file)
-	my ( $self, $name ) = @_;
-	my $context = $self->{context};
-	$context = Net::DNS::Domain->origin(undef) unless $context;
-	$self->{context} = &$context( sub { Net::DNS::Domain->origin($name) } );
-	delete $self->{latest};					# forget previous owner
-	return;
+	my $self = shift;
+	$self->{context} = origin Net::DNS::Domain(shift);
+	delete $self->{latest};					# forbid empty owner field
 }
 
 
@@ -607,7 +576,7 @@ All rights reserved.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the original copyright notices appear in all copies and that both
+that the above copyright notice appear in all copies and that both that
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
@@ -624,11 +593,8 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl> L<Net::DNS> L<Net::DNS::RR>
-L<RFC1035(5.1)|https://iana.org/go/rfc1035/section-5.1>
-L<RFC2308(4)|https://iana.org/go/rfc2308/section-4>
-
-L<BIND Administrator Reference Manual|https://bind9.readthedocs.io/en/latest/>
+L<perl>, L<Net::DNS>, L<Net::DNS::RR>, RFC1035 Section 5.1,
+RFC2308, BIND 9 Administrator Reference Manual
 
 =cut
 
